@@ -114,7 +114,7 @@ void ObjectGroup::moveObjects(int from, int to, int count)
 QRectF ObjectGroup::objectsBoundingRect() const
 {
     QRectF boundingRect;
-    foreach (const MapObject *object, mObjects)
+    for (const MapObject *object : mObjects)
         boundingRect = boundingRect.united(object->bounds());
     return boundingRect;
 }
@@ -128,7 +128,7 @@ QSet<SharedTileset> ObjectGroup::usedTilesets() const
 {
     QSet<SharedTileset> tilesets;
 
-    foreach (const MapObject *object, mObjects)
+    for (const MapObject *object : mObjects)
         if (const Tile *tile = object->cell().tile)
             tilesets.insert(tile->sharedTileset());
 
@@ -137,7 +137,7 @@ QSet<SharedTileset> ObjectGroup::usedTilesets() const
 
 bool ObjectGroup::referencesTileset(const Tileset *tileset) const
 {
-    foreach (const MapObject *object, mObjects) {
+    for (const MapObject *object : mObjects) {
         const Tile *tile = object->cell().tile;
         if (tile && tile->tileset() == tileset)
             return true;
@@ -149,11 +149,11 @@ bool ObjectGroup::referencesTileset(const Tileset *tileset) const
 void ObjectGroup::replaceReferencesToTileset(Tileset *oldTileset,
                                              Tileset *newTileset)
 {
-    foreach (MapObject *object, mObjects) {
+    for (MapObject *object : mObjects) {
         const Tile *tile = object->cell().tile;
         if (tile && tile->tileset() == oldTileset) {
             Cell cell = object->cell();
-            cell.tile = newTileset->tileAt(tile->id());
+            cell.tile = newTileset->findOrCreateTile(tile->id());
             object->setCell(cell);
         }
     }
@@ -163,7 +163,7 @@ void ObjectGroup::offsetObjects(const QPointF &offset,
                                 const QRectF &bounds,
                                 bool wrapX, bool wrapY)
 {
-    foreach (MapObject *object, mObjects) {
+    for (MapObject *object : mObjects) {
         const QPointF objectCenter = object->bounds().center();
         if (!bounds.contains(objectCenter))
             continue;
@@ -196,7 +196,7 @@ Layer *ObjectGroup::mergedWith(Layer *other) const
     const ObjectGroup *og = static_cast<ObjectGroup*>(other);
 
     ObjectGroup *merged = static_cast<ObjectGroup*>(clone());
-    foreach (const MapObject *mapObject, og->objects())
+    for (const MapObject *mapObject : og->objects())
         merged->addObject(mapObject->clone());
     return merged;
 }
@@ -211,10 +211,33 @@ Layer *ObjectGroup::clone() const
     return initializeClone(new ObjectGroup(mName, mX, mY, mWidth, mHeight));
 }
 
+/**
+ * Resets the ids of all objects to 0. Mostly used when new ids should be
+ * assigned after the object group has been cloned.
+ */
+void ObjectGroup::resetObjectIds()
+{
+    const QList<MapObject*> &objects = mObjects;
+    for (MapObject *object : objects)
+        object->resetId();
+}
+
+/**
+ * Returns the highest object id in use by this object group, or 0 if no object
+ * with assigned id exists.
+ */
+int ObjectGroup::highestObjectId() const
+{
+    int id = 0;
+    for (const MapObject *object : mObjects)
+        id = std::max(id, object->id());
+    return id;
+}
+
 ObjectGroup *ObjectGroup::initializeClone(ObjectGroup *clone) const
 {
     Layer::initializeClone(clone);
-    foreach (const MapObject *object, mObjects)
+    for (const MapObject *object : mObjects)
         clone->addObject(object->clone());
     clone->setColor(mColor);
     clone->setDrawOrder(mDrawOrder);
