@@ -27,7 +27,6 @@
 #include "compression.h"
 
 #include <QFile>
-#include <QFileInfo>
 #include <QSaveFile>
 
 using namespace Droidcraft;
@@ -72,12 +71,12 @@ Tiled::Map *DroidcraftPlugin::read(const QString &fileName)
 
     // Load
     for (int i = 0; i < 48 * 48; i++) {
-        unsigned char tileFile = uncompressed.at(i);
+        unsigned char tileId = uncompressed.at(i);
 
         int y = i / 48;
         int x = i - (48 * y);
 
-        Tile *tile = mapTileset->tileAt(tileFile);
+        Tile *tile = mapTileset->findTile(tileId);
         mapLayer->setCell(x, y, Cell(tile));
     }
 
@@ -88,7 +87,7 @@ Tiled::Map *DroidcraftPlugin::read(const QString &fileName)
 
 bool DroidcraftPlugin::supportsFile(const QString &fileName) const
 {
-    return QFileInfo(fileName).suffix() == QLatin1String("dat");
+    return fileName.endsWith(QLatin1String(".dat"), Qt::CaseInsensitive);
 }
 
 // Writer
