@@ -138,15 +138,15 @@ void LuaPlugin::writeMap(LuaTableWriter &writer, const Map *map)
 
     mGidMapper.clear();
     unsigned firstGid = 1;
-    foreach (const SharedTileset &tileset, map->tilesets()) {
+    for (const SharedTileset &tileset : map->tilesets()) {
         writeTileset(writer, tileset.data(), firstGid);
         mGidMapper.insert(firstGid, tileset.data());
-        firstGid += tileset->tileCount();
+        firstGid += tileset->nextTileId();
     }
     writer.writeEndTable();
 
     writer.writeStartTable("layers");
-    foreach (const Layer *layer, map->layers()) {
+    for (const Layer *layer : map->layers()) {
         switch (layer->layerType()) {
         case Layer::TileLayerType:
             writeTileLayer(writer,
@@ -173,8 +173,14 @@ void LuaPlugin::writeProperties(LuaTableWriter &writer,
 
     Properties::const_iterator it = properties.constBegin();
     Properties::const_iterator it_end = properties.constEnd();
-    for (; it != it_end; ++it)
-        writer.writeQuotedKeyAndValue(it.key(), it.value());
+    for (; it != it_end; ++it) {
+        QVariant value = toExportValue(it.value());
+
+        if (it.value().userType() == filePathTypeId())
+            value = mMapDir.relativeFilePath(value.toString());
+
+        writer.writeQuotedKeyAndValue(it.key(), value);
+    }
 
     writer.writeEndTable();
 }
@@ -254,15 +260,13 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
 
     writer.writeKeyAndValue("tilecount", tileset->tileCount());
     writer.writeStartTable("tiles");
-    for (int i = 0; i < tileset->tileCount(); ++i) {
-        const Tile *tile = tileset->tileAt(i);
-
+    for (const Tile *tile : tileset->tiles()) {
         // For brevity only write tiles with interesting properties
         if (!includeTile(tile))
             continue;
 
         writer.writeStartTable();
-        writer.writeKeyAndValue("id", i);
+        writer.writeKeyAndValue("id", tile->id());
 
         if (!tile->properties().isEmpty())
             writeProperties(writer, tile->properties());
@@ -297,7 +301,7 @@ void LuaPlugin::writeTileset(LuaTableWriter &writer, const Tileset *tileset,
             const QVector<Frame> &frames = tile->frames();
 
             writer.writeStartTable("animation");
-            foreach (const Frame &frame, frames) {
+            for (const Frame &frame : frames) {
                 writer.writeStartTable();
                 writer.writeKeyAndValue("tileid", QString::number(frame.tileId));
                 writer.writeKeyAndValue("duration", QString::number(frame.duration));
@@ -386,10 +390,12 @@ void LuaPlugin::writeObjectGroup(LuaTableWriter &writer,
     writer.writeKeyAndValue("offsetx", offset.x());
     writer.writeKeyAndValue("offsety", offset.y());
 
+    writer.writeKeyAndValue("draworder", drawOrderToString(objectGroup->drawOrder()));
+
     writeProperties(writer, objectGroup->properties());
 
     writer.writeStartTable("objects");
-    foreach (MapObject *mapObject, objectGroup->objects())
+    for (MapObject *mapObject : objectGroup->objects())
         writeMapObject(writer, mapObject);
     writer.writeEndTable();
 
@@ -403,11 +409,13 @@ void LuaPlugin::writeImageLayer(LuaTableWriter &writer,
 
     writer.writeKeyAndValue("type", "imagelayer");
     writer.writeKeyAndValue("name", imageLayer->name());
-    writer.writeKeyAndValue("x", imageLayer->x());
-    writer.writeKeyAndValue("y", imageLayer->y());
     writer.writeKeyAndValue("visible", imageLayer->isVisible());
     writer.writeKeyAndValue("opacity", imageLayer->opacity());
 
+    const QPointF offset = imageLayer->offset();
+    writer.writeKeyAndValue("offsetx", offset.x());
+    writer.writeKeyAndValue("offsety", offset.y());
+
     const QString rel = mMapDir.relativeFilePath(imageLayer->imageSource());
     writer.writeKeyAndValue("image", rel);
 
@@ -473,7 +481,7 @@ void LuaPlugin::writeMapObject(LuaTableWriter &writer,
          *    ...
          *  }
          */
-        foreach (const QPointF &point, polygon) {
+        for (const QPointF &point : polygon) {
             writer.writeStartTable();
             writer.setSuppressNewlines(true);
 
@@ -493,7 +501,7 @@ void LuaPlugin::writeMapObject(LuaTableWriter &writer,
          *    ...
          *  }
          */
-        foreach (const QPointF &point, polygon) {
+        for (const QPointF &point : polygon) {
             writer.writeStartTable();
             writer.setSuppressNewlines(true);
 
@@ -515,14 +523,14 @@ void LuaPlugin::writeMapObject(LuaTableWriter &writer,
 
         writer.writeStartTable("x");
         writer.setSuppressNewlines(true);
-        foreach (const QPointF &point, polygon)
+        for (const QPointF &point : polygon)
             writer.writeValue(point.x());
         writer.writeEndTable();
         writer.setSuppressNewlines(false);
 
         writer.writeStartTable("y");
         writer.setSuppressNewlines(true);
-        foreach (const QPointF &point, polygon)
+        for (const QPointF &point : polygon)
             writer.writeValue(point.y());
         writer.writeEndTable();
         writer.setSuppressNewlines(false);
