@@ -34,6 +34,9 @@ class QEvent;
 class QKeyEvent;
 
 namespace Tiled {
+
+class Layer;
+
 namespace Internal {
 
 class MapDocument;
@@ -157,6 +160,8 @@ protected:
 
     MapDocument *mapDocument() const { return mMapDocument; }
 
+    Layer *currentLayer() const;
+
 protected slots:
     /**
      * By default, this function is called after the current map has changed
