@@ -22,10 +22,12 @@
 #define CHANGEMAPOBJECT_H
 
 #include <QUndoCommand>
+#include <QVector>
 
 namespace Tiled {
 
 class MapObject;
+class Tile;
 
 namespace Internal {
 
@@ -76,6 +78,42 @@ private:
     bool mNewVisible;
 };
 
+
+struct MapObjectChange
+{
+    MapObject *object;
+
+    union {
+        Tile *tile;
+    };
+};
+
+class ChangeMapObjects : public QUndoCommand
+{
+public:
+    enum ChangeProperty {
+        ChangeTile
+    };
+
+    /**
+     * Creates an undo command that applies the given map object changes.
+     */
+    ChangeMapObjects(MapDocument *mapDocument,
+                     const QVector<MapObjectChange> &changes,
+                     ChangeProperty property,
+                     QUndoCommand *parent = nullptr);
+
+    void undo() override { swap(); }
+    void redo() override { swap(); }
+
+private:
+    void swap();
+
+    MapObjectModel *mMapObjectModel;
+    QVector<MapObjectChange> mChanges;
+    ChangeProperty mProperty;
+};
+
 } // namespace Internal
 } // namespace Tiled
 
