@@ -29,22 +29,42 @@ namespace Tiled {
 namespace Internal {
 
 ChangeTileProbability::ChangeTileProbability(MapDocument *mapDocument,
-                                             Tile *tile,
+                                             const QList<Tile*>& tiles,
                                              float probability)
     : mMapDocument(mapDocument)
-    , mTile(tile)
-    , mProbability(probability)
+    , mTiles(tiles)
 {
+    mProbabilities.reserve(tiles.size());
+    for (int i = 0; i < tiles.size(); ++ i)
+        mProbabilities.append(probability);
+
+    setText(QCoreApplication::translate("Undo Commands",
+                                        "Change Tile Probability"));
+}
+
+ChangeTileProbability::ChangeTileProbability(MapDocument *mapDocument,
+                                             const QList<Tile *> &tiles,
+                                             const QList<float> &probabilities,
+                                             QUndoCommand *parent)
+    : QUndoCommand(parent)
+    , mMapDocument(mapDocument)
+    , mTiles(tiles)
+    , mProbabilities(probabilities)
+{
+    Q_ASSERT(mTiles.size() == mProbabilities.size());
     setText(QCoreApplication::translate("Undo Commands",
                                         "Change Tile Probability"));
 }
 
 void ChangeTileProbability::swap()
 {
-    float probability = mTile->probability();
-    mTile->setProbability(mProbability);
-    mProbability = probability;
-    mMapDocument->emitTileProbabilityChanged(mTile);
+    for (int i = 0; i < mTiles.size(); ++ i) {
+        Tile* tile = mTiles[i];
+        float probability = tile->probability();
+        tile->setProbability(mProbabilities[i]);
+        mProbabilities[i] = probability;
+        mMapDocument->emitTileProbabilityChanged(tile);
+    }
 }
 
 } // namespace Internal
