@@ -35,9 +35,14 @@ class ChangeTileProbability : public QUndoCommand
 {
 public:
     ChangeTileProbability(MapDocument *mapDocument,
-                          Tile *tile,
+                          const QList<Tile*> &tiles,
                           float probability);
 
+    ChangeTileProbability(MapDocument *mapDocument,
+                          const QList<Tile*> &tiles,
+                          const QList<float> &probabilities,
+                          QUndoCommand *parent = nullptr);
+
     void undo() override { swap(); }
     void redo() override { swap(); }
 
@@ -45,8 +50,8 @@ private:
     void swap();
 
     MapDocument *mMapDocument;
-    Tile *mTile;
-    float mProbability;
+    QList<Tile*> mTiles;
+    QList<float> mProbabilities;
 };
 
 } // namespace Internal
