@@ -24,6 +24,7 @@
 #include "mapdocument.h"
 #include "mapobject.h"
 
+#include <QDir>
 #include <QMessageBox>
 #include <QSettings>
 
@@ -35,17 +36,22 @@ QString Command::finalCommand() const
     QString finalCommand = command;
 
     // Perform variable replacement
-    MapDocument *mapDocument = DocumentManager::instance()->currentDocument();
-    if (mapDocument) {
+    if (MapDocument *mapDocument = DocumentManager::instance()->currentDocument()) {
         const QString fileName = mapDocument->fileName();
 
         finalCommand.replace(QLatin1String("%mapfile"),
                              QString(QLatin1String("\"%1\"")).arg(fileName));
 
-        MapObject *currentObject = dynamic_cast<MapObject *>(mapDocument->currentObject());
-        if (currentObject) {
+        if (const Layer *layer = mapDocument->currentLayer()) {
+            finalCommand.replace(QLatin1String("%layername"),
+                                 QString(QLatin1String("\"%1\"")).arg(layer->name()));
+        }
+
+        if (MapObject *currentObject = dynamic_cast<MapObject *>(mapDocument->currentObject())) {
             finalCommand.replace(QLatin1String("%objecttype"),
                                  QString(QLatin1String("\"%1\"")).arg(currentObject->type()));
+            finalCommand.replace(QLatin1String("%objectid"),
+                                 QString(QLatin1String("\"%1\"")).arg(currentObject->id()));
         }
     }
 
@@ -100,7 +106,7 @@ CommandProcess::CommandProcess(const Command &command, bool inTerminal)
     , mName(command.name)
     , mFinalCommand(command.finalCommand())
 #ifdef Q_OS_MAC
-    , mFile(QLatin1String("tiledXXXXXX.command"))
+    , mFile(QDir::tempPath() + QLatin1String("/tiledXXXXXX.command"))
 #endif
 {
     // Give an error if the command is empty or just whitespace
@@ -132,7 +138,7 @@ CommandProcess::CommandProcess(const Command &command, bool inTerminal)
             handleError(tr("Unable to create/open %1").arg(mFile.fileName()));
             return;
         }
-        mFile.write(mFinalCommand.toStdString().c_str());
+        mFile.write(mFinalCommand.toLocal8Bit());
         mFile.close();
 
         // Add execute permission to the file
