@@ -110,10 +110,9 @@ void CommandDataModel::removeRows(QModelIndexList indices)
         mCommands.removeAt(row);
 
         // Decrement later indices since we removed a row
-        for (QModelIndexList::iterator i = indices.begin(); i != indices.end();
-                                                                            ++i)
-            if (i->row() > row)
-                *i = i->sibling(i->row() - 1, i->column());
+        for (QModelIndex &index : indices)
+            if (index.row() > row)
+                index = index.sibling(index.row() - 1, index.column());
 
         endRemoveRows();
     }
