@@ -1,7 +1,7 @@
 /*
  * documentmanager.cpp
  * Copyright 2010, Stefan Beller <stefanbeller@googlemail.com>
- * Copyright 2010, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
+ * Copyright 2010-2016, Thorbjørn Lindeijer <thorbjorn@lindeijer.nl>
  *
  * This file is part of Tiled.
  *
@@ -22,27 +22,71 @@
 #include "documentmanager.h"
 
 #include "abstracttool.h"
+#include "adjusttileindexes.h"
+#include "brokenlinks.h"
 #include "filesystemwatcher.h"
 #include "map.h"
 #include "mapdocument.h"
 #include "maprenderer.h"
 #include "mapscene.h"
 #include "mapview.h"
-#include "movabletabwidget.h"
+#include "tilesetmanager.h"
 #include "zoomable.h"
 
-#include <QUndoGroup>
 #include <QFileInfo>
+#include <QUndoGroup>
 
-#include <QVBoxLayout>
+#include <QApplication>
+#include <QClipboard>
+#include <QDialogButtonBox>
+#include <QDir>
 #include <QHBoxLayout>
 #include <QLabel>
-#include <QDialogButtonBox>
+#include <QMenu>
+#include <QMessageBox>
+#include <QProcess>
 #include <QScrollBar>
+#include <QTabBar>
+#include <QTabWidget>
+#include <QUndoStack>
+#include <QVBoxLayout>
 
 using namespace Tiled;
 using namespace Tiled::Internal;
 
+/*
+ * Code based on FileUtils::showInGraphicalShell from Qt Creator
+ * Copyright (C) 2016 The Qt Company Ltd.
+ * Used under the terms of the GNU General Public License version 3
+ */
+static void showInFileManager(const QString &fileName)
+{
+    // Mac, Windows support folder or file.
+#if defined(Q_OS_WIN)
+    QStringList param;
+    if (!QFileInfo(fileName).isDir())
+        param += QLatin1String("/select,");
+    param += QDir::toNativeSeparators(fileName);
+    QProcess::startDetached(QLatin1String("explorer.exe"), param);
+#elif defined(Q_OS_MAC)
+    QStringList scriptArgs;
+    scriptArgs << QLatin1String("-e")
+               << QString::fromLatin1("tell application \"Finder\" to reveal POSIX file \"%1\"")
+                                     .arg(fileName);
+    QProcess::execute(QLatin1String("/usr/bin/osascript"), scriptArgs);
+    scriptArgs.clear();
+    scriptArgs << QLatin1String("-e")
+               << QLatin1String("tell application \"Finder\" to activate");
+    QProcess::execute(QLatin1String("/usr/bin/osascript"), scriptArgs);
+#else
+    // We cannot select a file here, because xdg-open would open the file
+    // instead of the file browser...
+    QProcess::startDetached(QString(QLatin1String("xdg-open \"%1\""))
+                            .arg(QFileInfo(fileName).absolutePath()));
+#endif
+}
+
+
 namespace Tiled {
 namespace Internal {
 
@@ -85,22 +129,37 @@ class MapViewContainer : public QWidget
     Q_OBJECT
 
 public:
-    MapViewContainer(MapView *mapView, QWidget *parent = nullptr)
+    MapViewContainer(MapView *mapView,
+                     MapDocument *mapDocument,
+                     QWidget *parent = nullptr)
         : QWidget(parent)
         , mMapView(mapView)
         , mWarning(new FileChangedWarning)
+        , mBrokenLinksModel(new BrokenLinksModel(mapDocument, this))
+        , mBrokenLinksWidget(nullptr)
     {
         mWarning->setVisible(false);
 
-        QVBoxLayout *layout = new QVBoxLayout;
+        QVBoxLayout *layout = new QVBoxLayout(this);
         layout->setMargin(0);
         layout->setSpacing(0);
+
+        if (mBrokenLinksModel->hasBrokenLinks()) {
+            mBrokenLinksWidget = new BrokenLinksWidget(mBrokenLinksModel, this);
+            layout->addWidget(mBrokenLinksWidget);
+
+            connect(mBrokenLinksWidget, &BrokenLinksWidget::ignore,
+                    this, &MapViewContainer::deleteBrokenLinksWidget);
+        }
+
+        connect(mBrokenLinksModel, &BrokenLinksModel::hasBrokenLinksChanged,
+                this, &MapViewContainer::hasBrokenLinksChanged);
+
         layout->addWidget(mapView);
         layout->addWidget(mWarning);
-        setLayout(layout);
 
-        connect(mWarning, SIGNAL(reload()), SIGNAL(reload()));
-        connect(mWarning, SIGNAL(ignore()), mWarning, SLOT(hide()));
+        connect(mWarning, &FileChangedWarning::reload, this, &MapViewContainer::reload);
+        connect(mWarning, &FileChangedWarning::ignore, mWarning, &FileChangedWarning::hide);
     }
 
     MapView *mapView() const { return mMapView; }
@@ -111,9 +170,27 @@ public:
 signals:
     void reload();
 
+private slots:
+    void hasBrokenLinksChanged(bool hasBrokenLinks)
+    {
+        if (!hasBrokenLinks)
+            deleteBrokenLinksWidget();
+    }
+
+    void deleteBrokenLinksWidget()
+    {
+        if (mBrokenLinksWidget) {
+            mBrokenLinksWidget->deleteLater();
+            mBrokenLinksWidget = nullptr;
+        }
+    }
+
 private:
     MapView *mMapView;
+
     FileChangedWarning *mWarning;
+    BrokenLinksModel *mBrokenLinksModel;
+    BrokenLinksWidget *mBrokenLinksWidget;
 };
 
 } // namespace Internal
@@ -136,7 +213,7 @@ void DocumentManager::deleteInstance()
 
 DocumentManager::DocumentManager(QObject *parent)
     : QObject(parent)
-    , mTabWidget(new MovableTabWidget)
+    , mTabWidget(new QTabWidget)
     , mUndoGroup(new QUndoGroup(this))
     , mSelectedTool(nullptr)
     , mViewWithTool(nullptr)
@@ -144,16 +221,23 @@ DocumentManager::DocumentManager(QObject *parent)
 {
     mTabWidget->setDocumentMode(true);
     mTabWidget->setTabsClosable(true);
+    mTabWidget->setMovable(true);
+    mTabWidget->tabBar()->setContextMenuPolicy(Qt::CustomContextMenu);
 
     connect(mTabWidget, SIGNAL(currentChanged(int)),
             SLOT(currentIndexChanged()));
     connect(mTabWidget, SIGNAL(tabCloseRequested(int)),
             SIGNAL(documentCloseRequested(int)));
-    connect(mTabWidget, SIGNAL(tabMoved(int,int)),
+    connect(mTabWidget->tabBar(), SIGNAL(tabMoved(int,int)),
             SLOT(documentTabMoved(int,int)));
+    connect(mTabWidget->tabBar(), SIGNAL(customContextMenuRequested(QPoint)),
+            SLOT(tabContextMenuRequested(QPoint)));
 
     connect(mFileSystemWatcher, SIGNAL(fileChanged(QString)),
             SLOT(fileChanged(QString)));
+
+    connect(TilesetManager::instance(), &TilesetManager::tilesetChanged,
+            this, &DocumentManager::tilesetChanged);
 }
 
 DocumentManager::~DocumentManager()
@@ -262,7 +346,7 @@ void DocumentManager::addDocument(MapDocument *mapDocument)
 
     MapView *view = new MapView;
     MapScene *scene = new MapScene(view); // scene is owned by the view
-    MapViewContainer *container = new MapViewContainer(view, mTabWidget);
+    MapViewContainer *container = new MapViewContainer(view, mapDocument, mTabWidget);
 
     scene->setMapDocument(mapDocument);
     view->setScene(scene);
@@ -339,7 +423,7 @@ bool DocumentManager::reloadDocumentAt(int index)
     // Replace old tab
     addDocument(newDocument);
     closeDocumentAt(index);
-    mTabWidget->moveTab(mDocuments.size() - 1, index);
+    mTabWidget->tabBar()->moveTab(mDocuments.size() - 1, index);
 
     // Restore previous view state
     mapView = currentMapView();
@@ -349,6 +433,8 @@ bool DocumentManager::reloadDocumentAt(int index)
     if (layerIndex > 0 && layerIndex < newDocument->map()->layerCount())
         newDocument->setCurrentLayerIndex(layerIndex);
 
+    checkTilesetColumns(newDocument);
+
     return true;
 }
 
@@ -458,6 +544,31 @@ void DocumentManager::documentTabMoved(int from, int to)
     mDocuments.move(from, to);
 }
 
+void DocumentManager::tabContextMenuRequested(const QPoint &pos)
+{
+    QTabBar *tabBar = mTabWidget->tabBar();
+    int index = tabBar->tabAt(pos);
+    if (index == -1)
+        return;
+
+    QMenu menu(mTabWidget->window());
+
+    QString fileName = mDocuments.at(index)->fileName();
+
+    QAction *copyPath = menu.addAction(tr("Copy File Path"));
+    connect(copyPath, &QAction::triggered, [fileName] {
+        QClipboard *clipboard = QApplication::clipboard();
+        clipboard->setText(QDir::toNativeSeparators(fileName));
+    });
+
+    QAction *openFolder = menu.addAction(tr("Open Containing Folder..."));
+    connect(openFolder, &QAction::triggered, [fileName] {
+        showInFileManager(fileName);
+    });
+
+    menu.exec(tabBar->mapToGlobal(pos));
+}
+
 void DocumentManager::fileChanged(const QString &fileName)
 {
     const int index = findDocument(fileName);
@@ -498,11 +609,107 @@ void DocumentManager::cursorChanged(const QCursor &cursor)
 
 void DocumentManager::centerViewOn(qreal x, qreal y)
 {
-    MapView *view = currentMapView();
-    if (!view)
+    const int index = mTabWidget->currentIndex();
+    if (index == -1)
         return;
 
-    view->centerOn(currentDocument()->renderer()->pixelToScreenCoords(x, y));
+    MapView *view = currentMapView();
+    MapDocument *document = mDocuments.at(index);
+
+    view->centerOn(document->renderer()->pixelToScreenCoords(x, y));
+}
+
+static bool mayNeedColumnCountAdjustment(const Tileset &tileset)
+{
+    if (tileset.isCollection())
+        return false;
+    if (!tileset.imageLoaded())
+        return false;
+    if (tileset.columnCount() == tileset.expectedColumnCount())
+        return false;
+    if (tileset.columnCount() == 0 || tileset.expectedColumnCount() == 0)
+        return false;
+
+    return true;
+}
+
+void DocumentManager::tilesetChanged(Tileset *tileset)
+{
+    if (!mayNeedColumnCountAdjustment(*tileset))
+        return;
+
+    SharedTileset sharedTileset = tileset->sharedPointer();
+
+    bool anyRelevantMap = false;
+    for (MapDocument *document : mDocuments) {
+        if (document->map()->tilesets().contains(sharedTileset)) {
+            anyRelevantMap = true;
+            break;
+        }
+    }
+
+    if (anyRelevantMap && askForAdjustment(*tileset)) {
+        bool tilesetAdjusted = false;
+
+        for (MapDocument *mapDocument : mDocuments) {
+            Map *map = mapDocument->map();
+
+            if (map->tilesets().contains(sharedTileset)) {
+                auto command1 = new AdjustTileIndexes(mapDocument, *tileset);
+                mapDocument->undoStack()->beginMacro(command1->text());
+                mapDocument->undoStack()->push(command1);
+
+                if (!tilesetAdjusted) {
+                    auto command2 = new AdjustTileMetaData(mapDocument, *tileset);
+                    tilesetAdjusted = true;
+                    mapDocument->undoStack()->push(command2);
+                }
+
+                mapDocument->undoStack()->endMacro();
+            }
+        }
+    }
+
+    tileset->syncExpectedColumnCount();
+}
+
+/**
+ * Checks whether the number of columns in tileset image based tilesets matches
+ * with the expected amount. Offers to adjust tile indexes if not.
+ */
+void DocumentManager::checkTilesetColumns(MapDocument *mapDocument)
+{
+    for (const SharedTileset &tileset : mapDocument->map()->tilesets()) {
+        if (!mayNeedColumnCountAdjustment(*tileset))
+            continue;
+
+        if (askForAdjustment(*tileset)) {
+            auto command1 = new AdjustTileIndexes(mapDocument, *tileset);
+            auto command2 = new AdjustTileMetaData(mapDocument, *tileset);
+
+            mapDocument->undoStack()->beginMacro(command1->text());
+            mapDocument->undoStack()->push(command1);
+            mapDocument->undoStack()->push(command2);
+            mapDocument->undoStack()->endMacro();
+        }
+
+        tileset.data()->syncExpectedColumnCount();
+    }
+}
+
+bool DocumentManager::askForAdjustment(const Tileset &tileset)
+{
+    int r = QMessageBox::question(mTabWidget->window(),
+                                  tr("Tileset Columns Changed"),
+                                  tr("The number of tile columns in the tileset '%1' appears to have changed from %2 to %3. "
+                                     "Do you want to adjust tile references?")
+                                  .arg(tileset.name())
+                                  .arg(tileset.expectedColumnCount())
+                                  .arg(tileset.columnCount()),
+                                  QMessageBox::Yes | QMessageBox::No,
+                                  QMessageBox::Yes);
+
+    return r == QMessageBox::Yes;
 }
 
 #include "documentmanager.moc"
