@@ -27,6 +27,7 @@
 #include <QPair>
 #include <QPointF>
 
+class QTabWidget;
 class QUndoGroup;
 
 namespace Tiled {
@@ -40,7 +41,6 @@ class FileSystemWatcher;
 class MapDocument;
 class MapScene;
 class MapView;
-class MovableTabWidget;
 
 /**
  * This class controls the open documents.
@@ -144,6 +144,8 @@ public:
      */
     void closeAllDocuments();
 
+    void checkTilesetColumns(MapDocument *mapDocument);
+
     /**
      * Returns all open map documents.
      */
@@ -190,6 +192,7 @@ private slots:
     void updateDocumentTab();
     void documentSaved();
     void documentTabMoved(int from, int to);
+    void tabContextMenuRequested(const QPoint &pos);
 
     void fileChanged(const QString &fileName);
 
@@ -197,13 +200,17 @@ private slots:
 
     void cursorChanged(const QCursor &cursor);
 
+    void tilesetChanged(Tileset *tileset);
+
 private:
     DocumentManager(QObject *parent = nullptr);
     ~DocumentManager();
 
+    bool askForAdjustment(const Tileset &tileset);
+
     QList<MapDocument*> mDocuments;
 
-    MovableTabWidget *mTabWidget;
+    QTabWidget *mTabWidget;
     QUndoGroup *mUndoGroup;
     AbstractTool *mSelectedTool;
     MapView *mViewWithTool;
