@@ -28,14 +28,18 @@
 #include "mapreader.h"
 #include "mapformat.h"
 #include "preferences.h"
+#include "sparkleautoupdater.h"
+#include "standardautoupdater.h"
+#include "stylehelper.h"
 #include "tiledapplication.h"
 #include "tileset.h"
+#include "winsparkleautoupdater.h"
 
 #include <QDebug>
 #include <QFileInfo>
+#include <QJsonArray>
+#include <QJsonDocument>
 #include <QtPlugin>
-#include <QStyle>
-#include <QStyleFactory>
 
 #define STRINGIFY(x) #x
 #define AS_STRING(x) STRINGIFY(x)
@@ -56,6 +60,7 @@ public:
     bool showedVersion;
     bool disableOpenGL;
     bool exportMap;
+    bool newInstance;
 
 private:
     void showVersion();
@@ -63,6 +68,7 @@ private:
     void setDisableOpenGL();
     void setExportMap();
     void showExportFormats();
+    void startNewInstance();
 
     // Convenience wrapper around registerOption
     template <void (CommandLineHandler::*memberFunction)()>
@@ -85,6 +91,7 @@ CommandLineHandler::CommandLineHandler()
     , showedVersion(false)
     , disableOpenGL(false)
     , exportMap(false)
+    , newInstance(false)
 {
     option<&CommandLineHandler::showVersion>(
                 QLatin1Char('v'),
@@ -110,6 +117,11 @@ CommandLineHandler::CommandLineHandler()
                 QChar(),
                 QLatin1String("--export-formats"),
                 tr("Print a list of supported export formats"));
+
+    option<&CommandLineHandler::startNewInstance>(
+                QChar(),
+                QLatin1String("--new-instance"),
+                tr("Start a new instance, even if an instance is already running"));
 }
 
 void CommandLineHandler::showVersion()
@@ -151,8 +163,18 @@ void CommandLineHandler::showExportFormats()
     quit = true;
 }
 
+void CommandLineHandler::startNewInstance()
+{
+    newInstance = true;
+}
+
+
 int main(int argc, char *argv[])
 {
+#if QT_VERSION >= 0x050600
+    QGuiApplication::setFallbackSessionManagementEnabled(false);
+#endif
+
     TiledApplication a(argc, argv);
 
     a.setOrganizationDomain(QLatin1String("mapeditor.org"));
@@ -162,12 +184,7 @@ int main(int argc, char *argv[])
     a.setApplicationName(QLatin1String("tiled"));
 #endif
     a.setApplicationDisplayName(QLatin1String("Tiled"));
-
-#ifdef BUILD_INFO_VERSION
-    a.setApplicationVersion(QLatin1String(AS_STRING(BUILD_INFO_VERSION)));
-#else
-    a.setApplicationVersion(QLatin1String("0.14.2"));
-#endif
+    a.setApplicationVersion(QLatin1String(AS_STRING(TILED_VERSION)));
 
 #ifdef Q_OS_MAC
     a.setAttribute(Qt::AA_DontShowIconsInMenus);
@@ -176,23 +193,7 @@ int main(int argc, char *argv[])
     // Enable support for highres images (added in Qt 5.1, but off by default)
     a.setAttribute(Qt::AA_UseHighDpiPixmaps);
 
-#ifndef Q_OS_WIN
-    QString baseName = QApplication::style()->objectName();
-    if (baseName == QLatin1String("windows")) {
-        // Avoid Windows 95 style at all cost
-        if (QStyleFactory::keys().contains(QLatin1String("Fusion"))) {
-            baseName = QLatin1String("fusion"); // Qt5
-        } else { // Qt4
-            // e.g. if we are running on a KDE4 desktop
-            QByteArray desktopEnvironment = qgetenv("DESKTOP_SESSION");
-            if (desktopEnvironment == "kde")
-                baseName = QLatin1String("plastique");
-            else
-                baseName = QLatin1String("cleanlooks");
-        }
-        a.setStyle(QStyleFactory::create(baseName));
-    }
-#endif
+    StyleHelper::initialize();
 
     LanguageManager *languageManager = LanguageManager::instance();
     languageManager->installTranslators();
@@ -280,14 +281,36 @@ int main(int argc, char *argv[])
         return 0;
     }
 
+    if (!commandLine.filesToOpen().isEmpty() && !commandLine.newInstance) {
+        // Convert files to absolute paths because the already running Tiled
+        // instance likely does not have the same working directory.
+        QStringList absolutePaths;
+        for (const QString &fileName : commandLine.filesToOpen())
+            absolutePaths.append(QFileInfo(fileName).absoluteFilePath());
+        QJsonDocument doc(QJsonArray::fromStringList(absolutePaths));
+        if (a.sendMessage(QLatin1String(doc.toJson())))
+            return 0;
+    }
+
+    QScopedPointer<AutoUpdater> updater;
+#ifdef TILED_SPARKLE
+#if defined(Q_OS_MAC)
+    updater.reset(new SparkleAutoUpdater);
+#elif defined(Q_OS_WIN)
+    updater.reset(new WinSparkleAutoUpdater);
+#endif
+#endif
+
     MainWindow w;
     w.show();
 
+    a.setActivationWindow(&w);
+
     QObject::connect(&a, SIGNAL(fileOpenRequest(QString)),
                      &w, SLOT(openFile(QString)));
 
     if (!commandLine.filesToOpen().isEmpty()) {
-        foreach (const QString &fileName, commandLine.filesToOpen())
+        for (const QString &fileName : commandLine.filesToOpen())
             w.openFile(fileName);
     } else if (Preferences::instance()->openLastFilesOnStartup()) {
         w.openLastFiles();
