@@ -31,7 +31,6 @@
 #include "addremovemapobject.h"
 #include "automappingmanager.h"
 #include "addremovetileset.h"
-#include "clipboardmanager.h"
 #include "createobjecttool.h"
 #include "createrectangleobjecttool.h"
 #include "createellipseobjecttool.h"
@@ -63,12 +62,13 @@
 #include "resizedialog.h"
 #include "objectselectiontool.h"
 #include "objectgroup.h"
+#include "objecttypeseditor.h"
 #include "offsetmapdialog.h"
 #include "patreondialog.h"
 #include "preferences.h"
-#include "preferencesdialog.h"
 #include "propertiesdock.h"
 #include "stampbrush.h"
+#include "terrain.h"
 #include "terrainbrush.h"
 #include "tile.h"
 #include "tilelayer.h"
@@ -90,7 +90,7 @@
 #include "tileanimationeditor.h"
 #include "tilecollisioneditor.h"
 #include "tmxmapformat.h"
-#include "imagemovementtool.h"
+#include "layeroffsettool.h"
 #include "magicwandtool.h"
 #include "selectsametiletool.h"
 
@@ -98,28 +98,82 @@
 #include "macsupport.h"
 #endif
 
-#include <QMimeData>
 #include <QCloseEvent>
 #include <QComboBox>
 #include <QDesktopServices>
 #include <QFileDialog>
+#include <QIdentityProxyModel>
+#include <QLabel>
 #include <QMessageBox>
+#include <QMimeData>
+#include <QRegExp>
 #include <QScrollBar>
 #include <QSessionManager>
+#include <QShortcut>
+#include <QSignalMapper>
 #include <QTextStream>
+#include <QToolButton>
 #include <QUndoGroup>
 #include <QUndoStack>
 #include <QUndoView>
-#include <QImageReader>
-#include <QRegExp>
-#include <QSignalMapper>
-#include <QShortcut>
-#include <QToolButton>
 
 using namespace Tiled;
 using namespace Tiled::Internal;
 using namespace Tiled::Utils;
 
+namespace {
+
+/**
+ * A model that is always empty.
+ */
+class EmptyModel : public QAbstractListModel
+{
+public:
+    EmptyModel(QObject *parent = nullptr)
+        : QAbstractListModel(parent)
+    {}
+
+    int rowCount(const QModelIndex &) const override
+    { return 0; }
+
+    QVariant data(const QModelIndex &, int) const override
+    { return QVariant(); }
+};
+
+/**
+ * A proxy model that makes sure no items are checked or checkable.
+ *
+ * Used in the layer combo box, since the checkboxes can't be used in that
+ * context but are otherwise anyway rendered there on Windows.
+ */
+class UncheckableItemsModel : public QIdentityProxyModel
+{
+public:
+    UncheckableItemsModel(QObject *parent = nullptr)
+        : QIdentityProxyModel(parent)
+    {}
+
+    QVariant data(const QModelIndex &index, int role) const override
+    {
+        if (role == Qt::CheckStateRole)
+            return QVariant();
+
+        return QIdentityProxyModel::data(index, role);
+    }
+
+    Qt::ItemFlags flags(const QModelIndex &index) const override
+    {
+        Qt::ItemFlags rc = QIdentityProxyModel::flags(index);
+        return rc & ~Qt::ItemIsUserCheckable;
+    }
+};
+
+static EmptyModel emptyModel;
+static UncheckableItemsModel uncheckableLayerModel;
+
+} // anonymous namespace
+
+
 MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     : QMainWindow(parent, flags)
     , mUi(new Ui::MainWindow)
@@ -132,9 +186,10 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     , mTerrainDock(new TerrainDock(this))
     , mMiniMapDock(new MiniMapDock(this))
     , mConsoleDock(new ConsoleDock(this))
+    , mObjectTypesEditor(new ObjectTypesEditor(this))
     , mTileAnimationEditor(new TileAnimationEditor(this))
     , mTileCollisionEditor(new TileCollisionEditor(this))
-    , mCurrentLayerLabel(new QLabel)
+    , mLayerComboBox(new QComboBox)
     , mZoomable(nullptr)
     , mZoomComboBox(new QComboBox)
     , mStatusInfoLabel(new QLabel)
@@ -150,6 +205,10 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     MacSupport::addFullscreen(this);
 #endif
 
+#if QT_VERSION >= 0x050600
+    setDockOptions(dockOptions() | QMainWindow::GroupedDragging);
+#endif
+
     Preferences *preferences = Preferences::instance();
 
     QIcon redoIcon(QLatin1String(":images/16x16/edit-redo.png"));
@@ -179,7 +238,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     QAction *redoAction = undoGroup->createRedoAction(this, tr("Redo"));
     mUi->mainToolBar->setToolButtonStyle(Qt::ToolButtonFollowStyle);
     mUi->actionNew->setPriority(QAction::LowPriority);
-#if QT_VERSION >= 0x050500
+#if QT_VERSION == 0x050500
     undoAction->setPriority(QAction::LowPriority);
 #endif
     redoAction->setPriority(QAction::LowPriority);
@@ -190,11 +249,11 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(undoGroup, SIGNAL(cleanChanged(bool)), SLOT(updateWindowTitle()));
 
     UndoDock *undoDock = new UndoDock(undoGroup, this);
-    PropertiesDock *propertiesDock = new PropertiesDock(this);
+    mPropertiesDock = new PropertiesDock(this);
     TileStampsDock *tileStampsDock = new TileStampsDock(mTileStampManager, this);
 
     addDockWidget(Qt::RightDockWidgetArea, mLayerDock);
-    addDockWidget(Qt::LeftDockWidgetArea, propertiesDock);
+    addDockWidget(Qt::LeftDockWidgetArea, mPropertiesDock);
     addDockWidget(Qt::LeftDockWidgetArea, undoDock);
     addDockWidget(Qt::LeftDockWidgetArea, mMapsDock);
     addDockWidget(Qt::RightDockWidgetArea, mObjectsDock);
@@ -217,6 +276,12 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     mConsoleDock->setVisible(false);
     tileStampsDock->setVisible(false);
 
+    mLayerComboBox->setMinimumContentsLength(10);
+    mLayerComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);
+    connect(mLayerComboBox, SIGNAL(activated(int)),
+            this, SLOT(layerComboActivated(int)));
+
+    statusBar()->addPermanentWidget(mLayerComboBox);
     statusBar()->addPermanentWidget(mZoomComboBox);
 
     mUi->actionNew->setShortcuts(QKeySequence::New);
@@ -246,6 +311,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     mUi->actionSnapToGrid->setChecked(preferences->snapToGrid());
     mUi->actionSnapToFineGrid->setChecked(preferences->snapToFineGrid());
     mUi->actionHighlightCurrentLayer->setChecked(preferences->highlightCurrentLayer());
+    mUi->actionAutoMapWhileDrawing->setChecked(preferences->automappingDrawing());
 
     QActionGroup *objectLabelVisibilityGroup = new QActionGroup(this);
     mUi->actionNoLabels->setActionGroup(objectLabelVisibilityGroup);
@@ -343,6 +409,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mUi->actionCut, SIGNAL(triggered()), SLOT(cut()));
     connect(mUi->actionCopy, SIGNAL(triggered()), SLOT(copy()));
     connect(mUi->actionPaste, SIGNAL(triggered()), SLOT(paste()));
+    connect(mUi->actionPasteInPlace, SIGNAL(triggered()), SLOT(pasteInPlace()));
     connect(mUi->actionDelete, SIGNAL(triggered()), SLOT(delete_()));
     connect(mUi->actionPreferences, SIGNAL(triggered()),
             SLOT(openPreferences()));
@@ -362,16 +429,19 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mUi->actionZoomIn, SIGNAL(triggered()), SLOT(zoomIn()));
     connect(mUi->actionZoomOut, SIGNAL(triggered()), SLOT(zoomOut()));
     connect(mUi->actionZoomNormal, SIGNAL(triggered()), SLOT(zoomNormal()));
+    connect(mUi->actionFullScreen, &QAction::toggled, this, &MainWindow::setFullScreen);
 
     connect(mUi->actionNewTileset, SIGNAL(triggered()), SLOT(newTileset()));
     connect(mUi->actionAddExternalTileset, SIGNAL(triggered()),
             SLOT(addExternalTileset()));
     connect(mUi->actionResizeMap, SIGNAL(triggered()), SLOT(resizeMap()));
     connect(mUi->actionOffsetMap, SIGNAL(triggered()), SLOT(offsetMap()));
-    connect(mUi->actionMapProperties, SIGNAL(triggered()),
-            SLOT(editMapProperties()));
     connect(mUi->actionAutoMap, SIGNAL(triggered()),
             mAutomappingManager, SLOT(autoMap()));
+    connect(mUi->actionAutoMapWhileDrawing, &QAction::toggled,
+            preferences, &Preferences::setAutomappingDrawing);
+    connect(mUi->actionMapProperties, SIGNAL(triggered()),
+            SLOT(editMapProperties()));
 
     connect(mUi->actionDocumentation, SIGNAL(triggered()), SLOT(openDocumentation()));
     connect(mUi->actionBecomePatron, SIGNAL(triggered()), SLOT(becomePatron()));
@@ -381,13 +451,12 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
             SLOT(newTilesets(QStringList)));
 
     // Add recent file actions to the recent files menu
-    for (int i = 0; i < MaxRecentFiles; ++i)
-    {
-         mRecentFiles[i] = new QAction(this);
+    for (auto &action : mRecentFiles) {
+         action = new QAction(this);
          mUi->menuRecentFiles->insertAction(mUi->actionClearRecentFiles,
-                                            mRecentFiles[i]);
-         mRecentFiles[i]->setVisible(false);
-         connect(mRecentFiles[i], SIGNAL(triggered()),
+                                            action);
+         action->setVisible(false);
+         connect(action, SIGNAL(triggered()),
                  this, SLOT(openRecentFile()));
     }
     mUi->menuRecentFiles->insertSeparator(mUi->actionClearRecentFiles);
@@ -438,8 +507,12 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(mTilesetDock, SIGNAL(newTileset()),
             this, SLOT(newTileset()));
 
-    connect(mTerrainDock, SIGNAL(currentTerrainChanged(const Terrain*)),
-            this, SLOT(setTerrainBrush(const Terrain*)));
+    connect(mTerrainDock, &TerrainDock::currentTerrainChanged,
+            mTerrainBrush, &TerrainBrush::setTerrain);
+    connect(mTerrainDock, &TerrainDock::selectTerrainBrush,
+            this, &MainWindow::selectTerrainBrush);
+    connect(mTerrainBrush, &TerrainBrush::terrainCaptured,
+            mTerrainDock, &TerrainDock::setCurrentTerrain);
 
     connect(tileStampsDock, SIGNAL(setStamp(TileStamp)),
             this, SLOT(setStamp(TileStamp)));
@@ -466,7 +539,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     toolBar->addAction(mToolManager->registerTool(polylineObjectsTool));
     toolBar->addAction(mToolManager->registerTool(tileObjectsTool));
     toolBar->addSeparator();
-    toolBar->addAction(mToolManager->registerTool(new ImageMovementTool(this)));
+    toolBar->addAction(mToolManager->registerTool(new LayerOffsetTool(this)));
 
     mDocumentManager->setSelectedTool(mToolManager->selectedTool());
     connect(mToolManager, SIGNAL(selectedToolChanged(AbstractTool*)),
@@ -475,11 +548,12 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     statusBar()->addWidget(mStatusInfoLabel);
     connect(mToolManager, SIGNAL(statusInfoChanged(QString)),
             this, SLOT(updateStatusInfoLabel(QString)));
-    statusBar()->addWidget(mCurrentLayerLabel);
 
     // Add the 'Views and Toolbars' submenu. This needs to happen after all
     // the dock widgets and toolbars have been added to the main window.
     mViewsAndToolbarsMenu = new QAction(tr("Views and Toolbars"), this);
+    mShowObjectTypesEditor = new QAction(tr("Object Types Editor"), this);
+    mShowObjectTypesEditor->setCheckable(true);
     mShowTileAnimationEditor = new QAction(tr("Tile Animation Editor"), this);
     mShowTileAnimationEditor->setCheckable(true);
     mShowTileCollisionEditor = new QAction(tr("Tile Collision Editor"), this);
@@ -490,10 +564,15 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     popupMenu->setParent(this);
     mViewsAndToolbarsMenu->setMenu(popupMenu);
     mUi->menuView->insertAction(mUi->actionShowGrid, mViewsAndToolbarsMenu);
+    mUi->menuView->insertAction(mUi->actionShowGrid, mShowObjectTypesEditor);
     mUi->menuView->insertAction(mUi->actionShowGrid, mShowTileAnimationEditor);
     mUi->menuView->insertAction(mUi->actionShowGrid, mShowTileCollisionEditor);
     mUi->menuView->insertSeparator(mUi->actionShowGrid);
 
+    connect(mShowObjectTypesEditor, SIGNAL(toggled(bool)),
+            mObjectTypesEditor, SLOT(setVisible(bool)));
+    connect(mObjectTypesEditor, SIGNAL(closed()), SLOT(onObjectTypesEditorClosed()));
+
     connect(mShowTileAnimationEditor, SIGNAL(toggled(bool)),
             mTileAnimationEditor, SLOT(setVisible(bool)));
     connect(mTileAnimationEditor, SIGNAL(closed()), SLOT(onAnimationEditorClosed()));
@@ -525,6 +604,7 @@ MainWindow::MainWindow(QWidget *parent, Qt::WindowFlags flags)
     connect(switchToRightDocument1, SIGNAL(activated()),
             mDocumentManager, SLOT(switchToRightDocument()));
 
+    connect(qApp, &QApplication::commitDataRequest, this, &MainWindow::commitData);
 
     new QShortcut(tr("X"), this, SLOT(flipHorizontally()));
     new QShortcut(tr("Y"), this, SLOT(flipVertically()));
@@ -602,6 +682,9 @@ void MainWindow::changeEvent(QEvent *event)
         mUi->retranslateUi(this);
         retranslateUi();
         break;
+    case QEvent::WindowStateChange:
+        mUi->actionFullScreen->setChecked(isFullScreen());
+        break;
     default:
         break;
     }
@@ -666,6 +749,8 @@ bool MainWindow::openFile(const QString &fileName,
     }
 
     mDocumentManager->addDocument(mapDocument);
+    mDocumentManager->checkTilesetColumns(mapDocument);
+
     setRecentFile(fileName);
     return true;
 }
@@ -830,6 +915,21 @@ bool MainWindow::saveFileAs()
     if (fileName.isEmpty())
         return false;
 
+    if (!fileNameMatchesNameFilter(QFileInfo(fileName).fileName(), selectedFilter)) {
+        QMessageBox messageBox(QMessageBox::Warning,
+                               tr("Extension Mismatch"),
+                               tr("The file extension does not match the chosen file type."),
+                               QMessageBox::Yes | QMessageBox::No,
+                               window());
+
+        messageBox.setInformativeText(tr("Tiled may not automatically recognize your file when loading. "
+                                         "Are you sure you want to save with this extension?"));
+
+        int answer = messageBox.exec();
+        if (answer != QMessageBox::Yes)
+            return false;
+    }
+
     MapFormat *format = helper.formatByNameFilter(selectedFilter);
     mMapDocument->setWriterFormat(format);
 
@@ -1107,6 +1207,16 @@ void MainWindow::copy()
 }
 
 void MainWindow::paste()
+{
+    paste(ClipboardManager::PasteDefault);
+}
+
+void MainWindow::pasteInPlace()
+{
+    paste(ClipboardManager::PasteInPlace);
+}
+
+void MainWindow::paste(ClipboardManager::PasteFlags flags)
 {
     if (!mMapDocument)
         return;
@@ -1139,7 +1249,7 @@ void MainWindow::paste()
         mToolManager->selectTool(mStampBrush);
     } else if (ObjectGroup *objectGroup = layer->asObjectGroup()) {
         const MapView *view = mDocumentManager->currentMapView();
-        clipboardManager->pasteObjectGroup(objectGroup, mMapDocument, view);
+        clipboardManager->pasteObjectGroup(objectGroup, mMapDocument, view, flags);
     }
 
     if (map)
@@ -1175,8 +1285,14 @@ void MainWindow::delete_()
 
 void MainWindow::openPreferences()
 {
-    PreferencesDialog preferencesDialog(this);
-    preferencesDialog.exec();
+    if (!mPreferencesDialog) {
+        mPreferencesDialog = new PreferencesDialog(this);
+        mPreferencesDialog->setAttribute(Qt::WA_DeleteOnClose);
+    }
+
+    mPreferencesDialog->show();
+    mPreferencesDialog->activateWindow();
+    mPreferencesDialog->raise();
 }
 
 void MainWindow::labelVisibilityActionTriggered(QAction *action)
@@ -1209,6 +1325,17 @@ void MainWindow::zoomNormal()
         mapView->zoomable()->resetZoom();
 }
 
+void MainWindow::setFullScreen(bool fullScreen)
+{
+    if (isFullScreen() == fullScreen)
+        return;
+
+    if (fullScreen)
+        setWindowState(windowState() | Qt::WindowFullScreen);
+    else
+        setWindowState(windowState() & ~Qt::WindowFullScreen);
+}
+
 bool MainWindow::newTileset(const QString &path)
 {
     if (!mMapDocument)
@@ -1221,9 +1348,9 @@ bool MainWindow::newTileset(const QString &path)
             ? QFileInfo(prefs->lastPath(Preferences::ImageFile)).absolutePath()
             : path;
 
-    NewTilesetDialog newTileset(startLocation, this);
-    newTileset.setTileWidth(map->tileWidth());
-    newTileset.setTileHeight(map->tileHeight());
+    NewTilesetDialog newTileset(this);
+    newTileset.setImagePath(startLocation);
+    newTileset.setTileSize(map->tileSize());
 
     if (SharedTileset tileset = newTileset.createTileset()) {
         mMapDocument->undoStack()->push(new AddTileset(mMapDocument, tileset));
@@ -1281,7 +1408,7 @@ void MainWindow::addExternalTileset()
         return;
 
     prefs->setLastPath(Preferences::ExternalTileset,
-                       QFileInfo(fileNames.back()).path());
+                       QFileInfo(fileNames.last()).path());
 
     mSettings.setValue(QLatin1String("lastUsedTilesetFilter"), selectedFilter);
 
@@ -1329,7 +1456,7 @@ void MainWindow::resizeMap()
         const QSize &newSize = resizeDialog.newSize();
         const QPoint &offset = resizeDialog.offset();
         if (newSize != map->size() || !offset.isNull())
-            mMapDocument->resizeMap(newSize, offset);
+            mMapDocument->resizeMap(newSize, offset, resizeDialog.removeObjects());
     }
 }
 
@@ -1385,6 +1512,11 @@ void MainWindow::autoMappingWarning(bool automatic)
     }
 }
 
+void MainWindow::onObjectTypesEditorClosed()
+{
+    mShowObjectTypesEditor->setChecked(false);
+}
+
 void MainWindow::onAnimationEditorClosed()
 {
     mShowTileAnimationEditor->setChecked(false);
@@ -1395,6 +1527,19 @@ void MainWindow::onCollisionEditorClosed()
     mShowTileCollisionEditor->setChecked(false);
 }
 
+void MainWindow::layerComboActivated(int index)
+{
+    if (index == -1)
+        return;
+    if (!mMapDocument)
+        return;
+
+    int layerIndex = mMapDocument->layerModel()->toLayerIndex(index);
+
+    if (layerIndex != mMapDocument->currentLayerIndex())
+        mMapDocument->setCurrentLayerIndex(layerIndex);
+}
+
 void MainWindow::openRecentFile()
 {
     QAction *action = qobject_cast<QAction *>(sender());
@@ -1472,6 +1617,7 @@ void MainWindow::updateActions()
     bool tileLayerSelected = false;
     bool objectsSelected = false;
     QRegion selection;
+    int layerComboIndex = -1;
 
     if (mMapDocument) {
         Layer *currentLayer = mMapDocument->currentLayer();
@@ -1480,10 +1626,15 @@ void MainWindow::updateActions()
         tileLayerSelected = dynamic_cast<TileLayer*>(currentLayer) != nullptr;
         objectsSelected = !mMapDocument->selectedObjects().isEmpty();
         selection = mMapDocument->selectedArea();
+
+        int layerIndex = mMapDocument->currentLayerIndex();
+        if (layerIndex != -1)
+            layerComboIndex = mMapDocument->layerModel()->layerIndexToRow(layerIndex);
     }
 
     const bool canCopy = (tileLayerSelected && !selection.isEmpty())
             || objectsSelected;
+    const bool clipboardHasMap = ClipboardManager::instance()->hasMap();
 
     mUi->actionSave->setEnabled(map);
     mUi->actionSaveAs->setEnabled(map);
@@ -1496,7 +1647,8 @@ void MainWindow::updateActions()
     mUi->actionCloseAll->setEnabled(map);
     mUi->actionCut->setEnabled(canCopy);
     mUi->actionCopy->setEnabled(canCopy);
-    mUi->actionPaste->setEnabled(ClipboardManager::instance()->hasMap());
+    mUi->actionPaste->setEnabled(clipboardHasMap);
+    mUi->actionPasteInPlace->setEnabled(clipboardHasMap);
     mUi->actionDelete->setEnabled(canCopy);
     mUi->actionNewTileset->setEnabled(map);
     mUi->actionAddExternalTileset->setEnabled(map);
@@ -1509,9 +1661,8 @@ void MainWindow::updateActions()
 
     updateZoomLabel(); // for the zoom actions
 
-    Layer *layer = mMapDocument ? mMapDocument->currentLayer() : nullptr;
-    mCurrentLayerLabel->setText(tr("Current layer: %1").arg(
-                                    layer ? layer->name() : tr("<none>")));
+    mLayerComboBox->setEnabled(mMapDocument);
+    mLayerComboBox->setCurrentIndex(layerComboIndex);
 }
 
 void MainWindow::updateZoomLabel()
@@ -1583,17 +1734,9 @@ void MainWindow::setStamp(const TileStamp &stamp)
     mTilesetDock->selectTilesInStamp(stamp);
 }
 
-/**
- * Sets the terrain brush.
- */
-void MainWindow::setTerrainBrush(const Terrain *terrain)
+void MainWindow::selectTerrainBrush()
 {
-    mTerrainBrush->setTerrain(terrain);
-
-    // When selecting a new terrain, it makes sense to switch to a terrain brush tool
-    AbstractTool *selectedTool = mToolManager->selectedTool();
-    if (selectedTool != mTerrainBrush)
-        mToolManager->selectTool(mTerrainBrush);
+    mToolManager->selectTool(mTerrainBrush);
 }
 
 void MainWindow::updateStatusInfoLabel(const QString &statusInfo)
@@ -1603,6 +1746,12 @@ void MainWindow::updateStatusInfoLabel(const QString &statusInfo)
 
 void MainWindow::writeSettings()
 {
+#ifdef Q_OS_MAC
+    // See QTBUG-45241
+    if (isFullScreen())
+        setWindowState(windowState() & ~Qt::WindowFullScreen);
+#endif
+
     mSettings.beginGroup(QLatin1String("mainwindow"));
     mSettings.setValue(QLatin1String("geometry"), saveGeometry());
     mSettings.setValue(QLatin1String("state"), saveState());
@@ -1707,6 +1856,7 @@ void MainWindow::mapDocumentChanged(MapDocument *mapDocument)
 
     mActionHandler->setMapDocument(mapDocument);
     mLayerDock->setMapDocument(mapDocument);
+    mPropertiesDock->setMapDocument(mapDocument);
     mObjectsDock->setMapDocument(mapDocument);
     mTilesetDock->setMapDocument(mapDocument);
     mTerrainDock->setMapDocument(mapDocument);
@@ -1733,6 +1883,11 @@ void MainWindow::mapDocumentChanged(MapDocument *mapDocument)
             connect(mZoomable, SIGNAL(scaleChanged(qreal)),
                     this, SLOT(updateZoomLabel()));
         }
+
+        uncheckableLayerModel.setSourceModel(mapDocument->layerModel());
+        mLayerComboBox->setModel(&uncheckableLayerModel);
+    } else {
+        mLayerComboBox->setModel(&emptyModel);
     }
 
     updateWindowTitle();
