@@ -26,6 +26,8 @@
 
 #include "mapdocument.h"
 #include "consoledock.h"
+#include "clipboardmanager.h"
+#include "preferencesdialog.h"
 
 #include <QMainWindow>
 #include <QSessionManager>
@@ -57,6 +59,7 @@ class MapsDock;
 class MapView;
 class MiniMapDock;
 class ObjectsDock;
+class ObjectTypesEditor;
 class PropertiesDock;
 class StampBrush;
 class TerrainBrush;
@@ -130,6 +133,8 @@ private slots:
     void cut();
     void copy();
     void paste();
+    void pasteInPlace();
+    void paste(ClipboardManager::PasteFlags flags);
     void delete_(); // 'delete' is a reserved word
     void openPreferences();
 
@@ -137,6 +142,7 @@ private slots:
     void zoomIn();
     void zoomOut();
     void zoomNormal();
+    void setFullScreen(bool fullScreen);
 
     bool newTileset(const QString &path = QString());
     void newTilesets(const QStringList &paths);
@@ -164,7 +170,7 @@ private slots:
     void rotate(RotateDirection direction);
 
     void setStamp(const TileStamp &stamp);
-    void setTerrainBrush(const Terrain *terrain);
+    void selectTerrainBrush();
     void updateStatusInfoLabel(const QString &statusInfo);
 
     void mapDocumentChanged(MapDocument *mapDocument);
@@ -174,9 +180,12 @@ private slots:
     void autoMappingError(bool automatic);
     void autoMappingWarning(bool automatic);
 
+    void onObjectTypesEditorClosed();
     void onAnimationEditorClosed();
     void onCollisionEditorClosed();
 
+    void layerComboActivated(int index);
+
 private:
     /**
       * Asks the user whether the given \a mapDocument should be saved, when
@@ -220,15 +229,17 @@ private:
     MapDocument *mMapDocument;
     MapDocumentActionHandler *mActionHandler;
     LayerDock *mLayerDock;
+    PropertiesDock *mPropertiesDock;
     MapsDock *mMapsDock;
     ObjectsDock *mObjectsDock;
     TilesetDock *mTilesetDock;
     TerrainDock *mTerrainDock;
     MiniMapDock* mMiniMapDock;
     ConsoleDock *mConsoleDock;
+    ObjectTypesEditor *mObjectTypesEditor;
     TileAnimationEditor *mTileAnimationEditor;
     TileCollisionEditor *mTileCollisionEditor;
-    QLabel *mCurrentLayerLabel;
+    QComboBox *mLayerComboBox;
     Zoomable *mZoomable;
     QComboBox *mZoomComboBox;
     QLabel *mStatusInfoLabel;
@@ -245,6 +256,7 @@ private:
 
     QMenu *mLayerMenu;
     QAction *mViewsAndToolbarsMenu;
+    QAction *mShowObjectTypesEditor;
     QAction *mShowTileAnimationEditor;
     QAction *mShowTileCollisionEditor;
 
@@ -254,6 +266,8 @@ private:
     DocumentManager *mDocumentManager;
     ToolManager *mToolManager;
     TileStampManager *mTileStampManager;
+
+    QPointer<PreferencesDialog> mPreferencesDialog;
 };
 
 } // namespace Internal
