@@ -92,6 +92,8 @@ MapDocument::MapDocument(Map *map, const QString &fileName):
             SIGNAL(objectsAdded(QList<MapObject*>)));
     connect(mMapObjectModel, SIGNAL(objectsChanged(QList<MapObject*>)),
             SIGNAL(objectsChanged(QList<MapObject*>)));
+    connect(mMapObjectModel, SIGNAL(objectsTypeChanged(QList<MapObject*>)),
+            SIGNAL(objectsTypeChanged(QList<MapObject*>)));
     connect(mMapObjectModel, SIGNAL(objectsRemoved(QList<MapObject*>)),
             SLOT(onObjectsRemoved(QList<MapObject*>)));
 
@@ -318,7 +320,7 @@ static bool visibleIn(const QRectF &area, MapObject *object,
     return intersects(area, boundingRect);
 }
 
-void MapDocument::resizeMap(const QSize &size, const QPoint &offset)
+void MapDocument::resizeMap(const QSize &size, const QPoint &offset, bool removeObjects)
 {
     const QRegion movedSelection = mSelectedArea.translated(offset);
     const QRect newArea = QRect(-offset, size);
@@ -343,13 +345,15 @@ void MapDocument::resizeMap(const QSize &size, const QPoint &offset)
             ObjectGroup *objectGroup = static_cast<ObjectGroup*>(layer);
 
             // Remove objects that will fall outside of the map
-            foreach (MapObject *o, objectGroup->objects()) {
-                if (!visibleIn(visibleArea, o, mRenderer)) {
-                    mUndoStack->push(new RemoveMapObject(this, o));
-                } else {
-                    QPointF oldPos = o->position();
-                    QPointF newPos = oldPos + pixelOffset;
-                    mUndoStack->push(new MoveMapObject(this, o, newPos, oldPos));
+            if (removeObjects) {
+                foreach (MapObject *o, objectGroup->objects()) {
+                    if (!visibleIn(visibleArea, o, mRenderer)) {
+                        mUndoStack->push(new RemoveMapObject(this, o));
+                    } else {
+                        QPointF oldPos = o->position();
+                        QPointF newPos = oldPos + pixelOffset;
+                        mUndoStack->push(new MoveMapObject(this, o, newPos, oldPos));
+                    }
                 }
             }
             break;
@@ -380,7 +384,7 @@ void MapDocument::offsetMap(const QList<int> &layerIndexes,
                                          bounds, wrapX, wrapY));
     } else {
         mUndoStack->beginMacro(tr("Offset Map"));
-        foreach (const int layerIndex, layerIndexes) {
+        for (const int layerIndex : layerIndexes) {
             mUndoStack->push(new OffsetLayer(this, layerIndex, offset,
                                              bounds, wrapX, wrapY));
         }
@@ -474,6 +478,9 @@ void MapDocument::duplicateLayer()
     Layer *duplicate = mMap->layerAt(mCurrentLayerIndex)->clone();
     duplicate->setName(tr("Copy of %1").arg(duplicate->name()));
 
+    if (duplicate->layerType() == Layer::ObjectGroupType)
+        static_cast<ObjectGroup*>(duplicate)->resetObjectIds();
+
     const int index = mCurrentLayerIndex + 1;
     QUndoCommand *cmd = new AddLayer(this, index, duplicate);
     cmd->setText(tr("Duplicate Layer"));
@@ -615,6 +622,43 @@ void MapDocument::moveTileset(int from, int to)
     emit tilesetMoved(from, to);
 }
 
+/**
+ * Replaces the tileset at the given \a index with the new \a tileset. Replaces
+ * all tiles from the replaced tileset with tiles from the new tileset.
+ *
+ * @return The replaced tileset.
+ */
+SharedTileset MapDocument::replaceTileset(int index, const SharedTileset &tileset)
+{
+    SharedTileset oldTileset = mMap->tilesetAt(index);
+
+    mMap->replaceTileset(oldTileset, tileset);
+
+    // Try to keep the current object valid
+    if (mCurrentObject == oldTileset.data()) {
+        setCurrentObject(tileset.data());
+
+    } else if (mCurrentObject && mCurrentObject->typeId() == Object::TileType) {
+        Tile *tile = static_cast<Tile*>(mCurrentObject);
+        if (tile->tileset() == oldTileset.data())
+            setCurrentObject(tileset->findTile(tile->id()));
+
+    } else if (mCurrentObject && mCurrentObject->typeId() == Object::TerrainType) {
+        Terrain *terrain = static_cast<Terrain*>(mCurrentObject);
+        if (terrain->tileset() == oldTileset.data())
+            setCurrentObject(tileset->terrain(terrain->id()));
+    }
+
+
+    TilesetManager *tilesetManager = TilesetManager::instance();
+    tilesetManager->addReference(tileset);
+    tilesetManager->removeReference(oldTileset);
+
+    emit tilesetReplaced(index, tileset.data());
+
+    return oldTileset;
+}
+
 void MapDocument::setSelectedArea(const QRegion &selection)
 {
     if (mSelectedArea != selection) {
@@ -653,13 +697,13 @@ QList<Object*> MapDocument::currentObjects() const
     QList<Object*> objects;
     if (mCurrentObject) {
         if (mCurrentObject->typeId() == Object::MapObjectType && !mSelectedObjects.isEmpty()) {
-            for (MapObject *mapObj : mSelectedObjects) {
+            const auto &selectedObjects = mSelectedObjects;
+            for (MapObject *mapObj : selectedObjects)
                 objects.append(mapObj);
-            }
         } else if (mCurrentObject->typeId() == Object::TileType && !mSelectedTiles.isEmpty()) {
-            for (Tile *tile : mSelectedTiles) {
+            const auto &selectedTiles = mSelectedTiles;
+            for (Tile *tile : selectedTiles)
                 objects.append(tile);
-            }
         } else {
             objects.append(mCurrentObject);
         }
@@ -685,7 +729,7 @@ void MapDocument::unifyTilesets(Map *map)
     TilesetManager *tilesetManager = TilesetManager::instance();
 
     // Add tilesets that are not yet part of this map
-    foreach (const SharedTileset &tileset, map->tilesets()) {
+    for (const SharedTileset &tileset : map->tilesets()) {
         if (existingTilesets.contains(tileset))
             continue;
 
@@ -696,22 +740,23 @@ void MapDocument::unifyTilesets(Map *map)
         }
 
         // Merge the tile properties
-        const int sharedTileCount = qMin(tileset->tileCount(),
-                                         replacement->tileCount());
-        for (int i = 0; i < sharedTileCount; ++i) {
-            Tile *replacementTile = replacement->tileAt(i);
-            Properties properties = replacementTile->properties();
-            properties.merge(tileset->tileAt(i)->properties());
-            undoCommands.append(new ChangeProperties(this,
-                                                     tr("Tile"),
-                                                     replacementTile,
-                                                     properties));
+        for (Tile *replacementTile : replacement->tiles()) {
+            if (Tile *originalTile = tileset->findTile(replacementTile->id())) {
+                Properties properties = replacementTile->properties();
+                properties.merge(originalTile->properties());
+                undoCommands.append(new ChangeProperties(this,
+                                                         tr("Tile"),
+                                                         replacementTile,
+                                                         properties));
+            }
         }
-        map->replaceTileset(tileset, replacement);
 
         tilesetManager->addReference(replacement);
         tilesetManager->removeReference(tileset);
+
+        map->replaceTileset(tileset, replacement);
     }
+
     if (!undoCommands.isEmpty()) {
         mUndoStack->beginMacro(tr("Tileset Changes"));
         foreach (QUndoCommand *command, undoCommands)
@@ -733,7 +778,7 @@ void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTileset
     const QVector<SharedTileset> &existingTilesets = mMap->tilesets();
     TilesetManager *tilesetManager = TilesetManager::instance();
 
-    foreach (const SharedTileset &tileset, map->tilesets()) {
+    for (const SharedTileset &tileset : map->tilesets()) {
         // tileset already added
         if (existingTilesets.contains(tileset))
             continue;
@@ -748,10 +793,10 @@ void MapDocument::unifyTilesets(Map *map, QVector<SharedTileset> &missingTileset
         }
 
         // replacement tileset found, change given map
-        map->replaceTileset(tileset, replacement);
-
         tilesetManager->addReference(replacement);
         tilesetManager->removeReference(tileset);
+
+        map->replaceTileset(tileset, replacement);
     }
 }
 
@@ -843,15 +888,15 @@ void MapDocument::onLayerAboutToBeRemoved(int index)
 void MapDocument::onLayerRemoved(int index)
 {
     // Bring the current layer index to safety
-    bool currentLayerRemoved = mCurrentLayerIndex == mMap->layerCount();
-    if (currentLayerRemoved)
+    bool currentLayerAffected = index <= mCurrentLayerIndex;
+    if (currentLayerAffected)
         mCurrentLayerIndex = mCurrentLayerIndex - 1;
 
     emit layerRemoved(index);
 
     // Emitted after the layerRemoved signal so that the MapScene has a chance
     // of synchronizing before adapting to the newly selected index
-    if (currentLayerRemoved)
+    if (currentLayerAffected)
         emit currentLayerIndexChanged(mCurrentLayerIndex);
 }
 
@@ -905,8 +950,9 @@ void MapDocument::duplicateObjects(const QList<MapObject *> &objects)
     mUndoStack->beginMacro(tr("Duplicate %n Object(s)", "", objects.size()));
 
     QList<MapObject*> clones;
-    foreach (const MapObject *mapObject, objects) {
+    for (const MapObject *mapObject : objects) {
         MapObject *clone = mapObject->clone();
+        clone->resetId();
         clones.append(clone);
         mUndoStack->push(new AddMapObject(this,
                                           mapObject->objectGroup(),
@@ -950,9 +996,10 @@ void MapDocument::moveObjectsToGroup(const QList<MapObject *> &objects,
 
 void MapDocument::setProperty(Object *object,
                               const QString &name,
-                              const QString &value)
+                              const QVariant &value)
 {
     const bool hadProperty = object->hasProperty(name);
+
     object->setProperty(name, value);
 
     if (hadProperty)
