@@ -153,9 +153,10 @@ public:
 
     /**
      * Resize this map to the given \a size, while at the same time shifting
-     * the contents by \a offset.
+     * the contents by \a offset. If \a removeObjects is true then all objects
+     * which are outside the map will be removed.
      */
-    void resizeMap(const QSize &size, const QPoint &offset);
+    void resizeMap(const QSize &size, const QPoint &offset, bool removeObjects);
 
     /**
      * Offsets the layers at \a layerIndexes by \a offset, within \a bounds,
@@ -180,6 +181,7 @@ public:
     void insertTileset(int index, const SharedTileset &tileset);
     void removeTilesetAt(int index);
     void moveTileset(int from, int to);
+    SharedTileset replaceTileset(int index, const SharedTileset &tileset);
     void setTilesetFileName(Tileset *tileset, const QString &fileName);
     void setTilesetName(Tileset *tileset, const QString &name);
     void setTilesetTileOffset(Tileset *tileset, const QPoint &tileOffset);
@@ -189,7 +191,7 @@ public:
     void moveObjectsToGroup(const QList<MapObject*> &objects,
                             ObjectGroup *objectGroup);
 
-    void setProperty(Object *object, const QString &name, const QString &value);
+    void setProperty(Object *object, const QString &name, const QVariant &value);
     void setProperties(Object *object, const Properties &properties);
     void removeProperty(Object *object, const QString &name);
 
@@ -289,7 +291,7 @@ signals:
      * selected region and the previously selected region.
      */
     void selectedAreaChanged(const QRegion &newSelection,
-                              const QRegion &oldSelection);
+                             const QRegion &oldSelection);
 
     /**
      * Emitted when the list of selected objects changes.
@@ -342,6 +344,7 @@ signals:
 
     void tileLayerDrawMarginsChanged(TileLayer *layer);
 
+    void tileImageSourceChanged(Tile *tile);
     void tileTerrainChanged(const QList<Tile*> &tiles);
     void tileProbabilityChanged(Tile *tile);
     void tileObjectGroupChanged(Tile *tile);
@@ -356,6 +359,7 @@ signals:
     void tilesetAboutToBeRemoved(int index);
     void tilesetRemoved(Tileset *tileset);
     void tilesetMoved(int from, int to);
+    void tilesetReplaced(int index, Tileset *tileset);
     void tilesetFileNameChanged(Tileset *tileset);
     void tilesetNameChanged(Tileset *tileset);
     void tilesetTileOffsetChanged(Tileset *tileset);
@@ -365,6 +369,7 @@ signals:
     void objectsInserted(ObjectGroup *objectGroup, int first, int last);
     void objectsRemoved(const QList<MapObject*> &objects);
     void objectsChanged(const QList<MapObject*> &objects);
+    void objectsTypeChanged(const QList<MapObject*> &objects);
     void objectsIndexChanged(ObjectGroup *objectGroup, int first, int last);
 
     void propertyAdded(Object *object, const QString &name);
@@ -414,6 +419,35 @@ private:
     QDateTime mLastSaved;
 };
 
+
+/**
+ * Class used to describe an embedded Tileset, changes to which are applied
+ * using undo commands on the undo stack of the associated MapDocument.
+ */
+class EmbeddedTileset
+{
+public:
+    EmbeddedTileset()
+        : mMapDocument(nullptr)
+        , mTileset(nullptr)
+    {}
+
+    EmbeddedTileset(MapDocument *mapDocument, Tileset *tileset)
+        : mMapDocument(mapDocument)
+        , mTileset(tileset)
+        , mImageSource(tileset->imageSource())
+    {}
+
+    MapDocument *mapDocument() const { return mMapDocument; }
+    Tileset *tileset() const { return mTileset; }
+
+private:
+    MapDocument *mMapDocument;
+    Tileset *mTileset;
+    QString mImageSource;    // affects display
+};
+
+
 inline QString MapDocument::lastExportFileName() const
 {
     return mLastExportFileName;
@@ -530,4 +564,6 @@ inline void MapDocument::emitEditCurrentObject()
 } // namespace Internal
 } // namespace Tiled
 
+Q_DECLARE_METATYPE(Tiled::Internal::EmbeddedTileset)
+
 #endif // MAPDOCUMENT_H
