@@ -40,13 +40,15 @@
 #include "tileselectionitem.h"
 #include "imagelayer.h"
 #include "imagelayeritem.h"
+#include "stylehelper.h"
 #include "toolmanager.h"
 #include "tilesetmanager.h"
 
+#include <QApplication>
 #include <QGraphicsSceneMouseEvent>
 #include <QPainter>
 #include <QKeyEvent>
-#include <QApplication>
+#include <QPalette>
 
 #include <cmath>
 
@@ -64,10 +66,12 @@ MapScene::MapScene(QObject *parent):
     mUnderMouse(false),
     mCurrentModifiers(Qt::NoModifier),
     mDarkRectangle(new QGraphicsRectItem),
-    mDefaultBackgroundColor(Qt::darkGray),
     mObjectSelectionItem(nullptr)
 {
-    setBackgroundBrush(mDefaultBackgroundColor);
+    updateDefaultBackgroundColor();
+
+    connect(StyleHelper::instance(), &StyleHelper::styleApplied,
+            this, &MapScene::updateDefaultBackgroundColor);
 
     TilesetManager *tilesetManager = TilesetManager::instance();
     connect(tilesetManager, SIGNAL(tilesetChanged(Tileset*)),
@@ -142,8 +146,12 @@ void MapScene::setMapDocument(MapDocument *mapDocument)
                 this, SLOT(imageLayerChanged(ImageLayer*)));
         connect(mMapDocument, SIGNAL(currentLayerIndexChanged(int)),
                 this, SLOT(currentLayerIndexChanged()));
-        connect(mMapDocument, SIGNAL(tilesetTileOffsetChanged(Tileset*)),
-                this, SLOT(tilesetTileOffsetChanged(Tileset*)));
+        connect(mMapDocument, &MapDocument::tilesetTileOffsetChanged,
+                this, &MapScene::adaptToTilesetTileSizeChanges);
+        connect(mMapDocument, &MapDocument::tileImageSourceChanged,
+                this, &MapScene::adaptToTileSizeChanges);
+        connect(mMapDocument, &MapDocument::tilesetReplaced,
+                this, &MapScene::tilesetReplaced);
         connect(mMapDocument, SIGNAL(objectsInserted(ObjectGroup*,int,int)),
                 this, SLOT(objectsInserted(ObjectGroup*,int,int)));
         connect(mMapDocument, SIGNAL(objectsRemoved(QList<MapObject*>)),
@@ -252,6 +260,14 @@ QGraphicsItem *MapScene::createLayerItem(Layer *layer)
     return layerItem;
 }
 
+void MapScene::updateDefaultBackgroundColor()
+{
+    mDefaultBackgroundColor = QGuiApplication::palette().dark().color();
+
+    if (!mMapDocument || !mMapDocument->map()->backgroundColor().isValid())
+        setBackgroundBrush(mDefaultBackgroundColor);
+}
+
 void MapScene::updateSceneRect()
 {
     const QSize mapSize = mMapDocument->renderer()->mapSize();
@@ -460,10 +476,10 @@ void MapScene::imageLayerChanged(ImageLayer *imageLayer)
 }
 
 /**
- * When the tile offset of a tileset has changed, it can affect the bounding
- * rect of all tile layers and tile objects. It also requires a full repaint.
+ * This function should be called when any tiles in the given tileset may have
+ * changed their size or offset or image.
  */
-void MapScene::tilesetTileOffsetChanged(Tileset *tileset)
+void MapScene::adaptToTilesetTileSizeChanges(Tileset *tileset)
 {
     update();
 
@@ -478,6 +494,27 @@ void MapScene::tilesetTileOffsetChanged(Tileset *tileset)
     }
 }
 
+void MapScene::adaptToTileSizeChanges(Tile *tile)
+{
+    update();
+
+    for (QGraphicsItem *item : mLayerItems)
+        if (TileLayerItem *tli = dynamic_cast<TileLayerItem*>(item))
+            tli->syncWithTileLayer();
+
+    for (MapObjectItem *item : mObjectItems) {
+        const Cell &cell = item->mapObject()->cell();
+        if (cell.tile == tile)
+            item->syncWithMapObject();
+    }
+}
+
+void MapScene::tilesetReplaced(int index, Tileset *tileset)
+{
+    Q_UNUSED(index)
+    adaptToTilesetTileSizeChanges(tileset);
+}
+
 /**
  * Inserts map object items for the given objects.
  */
