@@ -117,7 +117,7 @@ void MiniMap::paintEvent(QPaintEvent *pe)
     QPainter p(this);
     p.setRenderHints(QPainter::SmoothPixmapTransform);
 
-    QColor backgroundColor(Qt::darkGray);
+    QColor backgroundColor(palette().dark().color());
     if (mMapDocument && mMapDocument->map()->backgroundColor().isValid())
         backgroundColor = mMapDocument->map()->backgroundColor();
     p.setBrush(backgroundColor);
@@ -280,7 +280,7 @@ void MiniMap::centerViewOnLocalPixel(QPoint centerPos, int delta)
     if (delta != 0)
         mapView->zoomable()->handleWheelDelta(delta);
 
-    mapView->centerOn(mapToScene(centerPos));
+    mapView->forceCenterOn(mapToScene(centerPos));
 }
 
 void MiniMap::redrawTimeout()
