@@ -32,9 +32,11 @@ class QGraphicsItem;
 namespace Tiled {
 namespace Internal {
 
-class RotateHandle;
-class ResizeHandle;
+class Handle;
 class MapObjectItem;
+class OriginIndicator;
+class ResizeHandle;
+class RotateHandle;
 class SelectionRectangle;
 
 class ObjectSelectionTool : public AbstractObjectTool
@@ -59,7 +61,7 @@ public:
     void languageChanged() override;
 
 private slots:
-    void updateHandles();
+    void updateHandles(bool resetOriginIndicator = true);
     void updateHandleVisibility();
 
     void objectsRemoved(const QList<MapObject *> &);
@@ -69,6 +71,7 @@ private:
         NoAction,
         Selecting,
         Moving,
+        MovingOrigin,
         Rotating,
         Resizing
     };
@@ -78,17 +81,22 @@ private:
         Rotate,
     };
 
+    void updateHover(const QPointF &pos);
     void updateSelection(const QPointF &pos,
                          Qt::KeyboardModifiers modifiers);
 
     void startSelecting();
 
-    void startMoving(Qt::KeyboardModifiers modifiers);
+    void startMoving(const QPointF &pos, Qt::KeyboardModifiers modifiers);
     void updateMovingItems(const QPointF &pos,
                            Qt::KeyboardModifiers modifiers);
     void finishMoving(const QPointF &pos);
 
-    void startRotating();
+    void startMovingOrigin(const QPointF &pos);
+    void updateMovingOrigin(const QPointF &pos, Qt::KeyboardModifiers modifiers);
+    void finishMovingOrigin();
+
+    void startRotating(const QPointF &pos);
     void updateRotatingItems(const QPointF &pos,
                              Qt::KeyboardModifiers modifiers);
     void finishRotating(const QPointF &pos);
@@ -127,13 +135,19 @@ private:
     RotateHandle *mRotateHandles[4];
     ResizeHandle *mResizeHandles[8];
     bool mMousePressed;
+
     MapObjectItem *mHoveredObjectItem;
+    Handle *mHoveredHandle;
+
     MapObjectItem *mClickedObjectItem;
+    OriginIndicator *mClickedOriginIndicator;
     RotateHandle *mClickedRotateHandle;
     ResizeHandle *mClickedResizeHandle;
 
     QVector<MovingObject> mMovingObjects;
 
+    QPointF mOldOriginPosition;
+
     QPointF mAlignPosition;
     QPointF mOrigin;
     bool mResizingLimitHorizontal;
@@ -141,6 +155,7 @@ private:
     Mode mMode;
     Action mAction;
     QPointF mStart;
+    QPointF mStartOffset;
     QPoint mScreenStart;
     Qt::KeyboardModifiers mModifiers;
 };
