@@ -32,8 +32,6 @@ class PreferencesDialog;
 namespace Tiled {
 namespace Internal {
 
-class ObjectTypesModel;
-
 /**
  * The preferences dialog. Allows the user to configure some general behaviour
  * settings of Tiled and choose the language.
@@ -51,26 +49,19 @@ protected:
 
 private slots:
     void languageSelected(int index);
-    void objectLineWidthChanged(double lineWidth);
-    void useOpenGLToggled(bool useOpenGL);
-    void useAutomappingDrawingToggled(bool enabled);
-    void openLastFilesToggled(bool enabled);
-
-    void addObjectType();
-    void selectedObjectTypesChanged();
-    void removeSelectedObjectTypes();
-    void objectTypeIndexClicked(const QModelIndex &index);
-    void applyObjectTypes();
-    void importObjectTypes();
-    void exportObjectTypes();
 
 private:
     void fromPreferences();
-    void toPreferences();
+
+    void retranslateUi();
+
+    void styleComboChanged(int index);
+
+    void autoUpdateToggled(bool checked);
+    void checkForUpdates();
 
     Ui::PreferencesDialog *mUi;
     QStringList mLanguages;
-    ObjectTypesModel *mObjectTypesModel;
 };
 
 
