@@ -27,6 +27,7 @@
 #include "changemapproperty.h"
 #include "changeobjectgroupproperties.h"
 #include "changeproperties.h"
+#include "changetileimagesource.h"
 #include "changetileprobability.h"
 #include "flipmapobjects.h"
 #include "imagelayer.h"
@@ -36,6 +37,7 @@
 #include "movemapobject.h"
 #include "objectgroup.h"
 #include "preferences.h"
+#include "replacetileset.h"
 #include "resizemapobject.h"
 #include "renamelayer.h"
 #include "renameterrain.h"
@@ -45,6 +47,8 @@
 #include "tile.h"
 #include "tilelayer.h"
 #include "tilesetchanges.h"
+#include "tilesetformat.h"
+#include "tmxmapformat.h"
 #include "utils.h"
 #include "varianteditorfactory.h"
 #include "variantpropertymanager.h"
@@ -52,6 +56,8 @@
 #include <QtGroupPropertyManager>
 
 #include <QCoreApplication>
+#include <QDebug>
+#include <QMessageBox>
 
 namespace Tiled {
 namespace Internal {
@@ -65,7 +71,9 @@ PropertyBrowser::PropertyBrowser(QWidget *parent)
     , mGroupManager(new QtGroupPropertyManager(this))
     , mCustomPropertiesGroup(nullptr)
 {
-    setFactoryForManager(mVariantManager, new VariantEditorFactory(this));
+    VariantEditorFactory *variantEditorFactory = new VariantEditorFactory(this);
+
+    setFactoryForManager(mVariantManager, variantEditorFactory);
     setResizeMode(ResizeToContents);
     setRootIsDecorated(false);
     setPropertiesWithoutValueMarked(true);
@@ -100,6 +108,9 @@ PropertyBrowser::PropertyBrowser(QWidget *parent)
 
     connect(mVariantManager, SIGNAL(valueChanged(QtProperty*,QVariant)),
             SLOT(valueChanged(QtProperty*,QVariant)));
+
+    connect(variantEditorFactory, &VariantEditorFactory::resetProperty,
+            this, &PropertyBrowser::resetProperty);
 }
 
 void PropertyBrowser::setObject(Object *object)
@@ -130,6 +141,8 @@ void PropertyBrowser::setMapDocument(MapDocument *mapDocument)
                 SLOT(mapChanged()));
         connect(mapDocument, SIGNAL(objectsChanged(QList<MapObject*>)),
                 SLOT(objectsChanged(QList<MapObject*>)));
+        connect(mapDocument, SIGNAL(objectsTypeChanged(QList<MapObject*>)),
+                SLOT(objectsTypeChanged(QList<MapObject*>)));
         connect(mapDocument, SIGNAL(layerChanged(int)),
                 SLOT(layerChanged(int)));
         connect(mapDocument, SIGNAL(objectGroupChanged(ObjectGroup*)),
@@ -137,13 +150,19 @@ void PropertyBrowser::setMapDocument(MapDocument *mapDocument)
         connect(mapDocument, SIGNAL(imageLayerChanged(ImageLayer*)),
                 SLOT(imageLayerChanged(ImageLayer*)));
 
-        connect(mapDocument, SIGNAL(tilesetNameChanged(Tileset*)),
-                SLOT(tilesetChanged(Tileset*)));
-        connect(mapDocument, SIGNAL(tilesetTileOffsetChanged(Tileset*)),
-                SLOT(tilesetChanged(Tileset*)));
+        connect(mapDocument, &MapDocument::tilesetFileNameChanged,
+                this, &PropertyBrowser::tilesetChanged);
+        connect(mapDocument, &MapDocument::tilesetNameChanged,
+                this, &PropertyBrowser::tilesetChanged);
+        connect(mapDocument, &MapDocument::tilesetTileOffsetChanged,
+                this, &PropertyBrowser::tilesetChanged);
+        connect(mapDocument, &MapDocument::tilesetChanged,
+                this, &PropertyBrowser::tilesetChanged);
 
-        connect(mapDocument, SIGNAL(tileProbabilityChanged(Tile*)),
-                SLOT(tileChanged(Tile*)));
+        connect(mapDocument, &MapDocument::tileProbabilityChanged,
+                this, &PropertyBrowser::tileChanged);
+        connect(mapDocument, &MapDocument::tileImageSourceChanged,
+                this, &PropertyBrowser::tileChanged);
 
         TerrainModel *terrainModel = mapDocument->terrainModel();
         connect(terrainModel, SIGNAL(terrainChanged(Tileset*,int)),
@@ -162,6 +181,9 @@ void PropertyBrowser::setMapDocument(MapDocument *mapDocument)
                 SLOT(selectedObjectsChanged()));
         connect(mapDocument, SIGNAL(selectedTilesChanged()),
                 SLOT(selectedTilesChanged()));
+
+        connect(Preferences::instance(), &Preferences::objectTypesChanged,
+                this, &PropertyBrowser::objectTypesChanged);
     }
 }
 
@@ -202,6 +224,13 @@ void PropertyBrowser::objectsChanged(const QList<MapObject *> &objects)
             updateProperties();
 }
 
+void PropertyBrowser::objectsTypeChanged(const QList<MapObject *> &objects)
+{
+    if (mObject && mObject->typeId() == Object::MapObjectType)
+        if (objects.contains(static_cast<MapObject*>(mObject)))
+            updateCustomProperties();
+}
+
 void PropertyBrowser::layerChanged(int index)
 {
     if (mObject == mMapDocument->map()->layerAt(index))
@@ -222,8 +251,18 @@ void PropertyBrowser::imageLayerChanged(ImageLayer *imageLayer)
 
 void PropertyBrowser::tilesetChanged(Tileset *tileset)
 {
-    if (mObject == tileset)
+    if (mObject != tileset)
+        return;
+
+    bool isExternal = tileset->isExternal();
+    bool wasExternal = mIdToProperty.contains(FileNameProperty);
+
+    if (isExternal == wasExternal) {
         updateProperties();
+    } else {
+        removeProperties();
+        addProperties();
+    }
 }
 
 void PropertyBrowser::tileChanged(Tile *tile)
@@ -255,11 +294,15 @@ void PropertyBrowser::propertyAdded(Object *object, const QString &name)
         QtProperty *precedingProperty = (index > 0) ? properties.at(index - 1) : nullptr;
 
         mUpdating = true;
-        QtVariantProperty *property = mVariantManager->addProperty(QVariant::String, name);
-        property->setValue(mObject->property(name));
+        QVariant value = object->property(name);
+        QtVariantProperty *property = createProperty(CustomProperty, value.userType(), name);
+        property->setValue(value);
         mCustomPropertiesGroup->insertSubProperty(property, precedingProperty);
-        mPropertyToId.insert(property, CustomProperty);
-        mNameToProperty.insert(name, property);
+
+        // Collapse custom color properties, to save space
+        if (value.type() == QVariant::Color)
+            setExpanded(items(property).first(), false);
+
         mUpdating = false;
     }
     updatePropertyColor(name);
@@ -317,6 +360,12 @@ void PropertyBrowser::selectedTilesChanged()
     updateCustomProperties();
 }
 
+void PropertyBrowser::objectTypesChanged()
+{
+    if (mObject && mObject->typeId() == Object::MapObjectType)
+        updateCustomProperties();
+}
+
 void PropertyBrowser::valueChanged(QtProperty *property, const QVariant &val)
 {
     if (mUpdating)
@@ -333,7 +382,7 @@ void PropertyBrowser::valueChanged(QtProperty *property, const QVariant &val)
         undoStack->push(new SetProperty(mMapDocument,
                                         mMapDocument->currentObjects(),
                                         property->propertyName(),
-                                        val.toString()));
+                                        val));
         return;
     }
 
@@ -347,58 +396,71 @@ void PropertyBrowser::valueChanged(QtProperty *property, const QVariant &val)
     }
 }
 
+void PropertyBrowser::resetProperty(QtProperty *property)
+{
+    switch (mVariantManager->propertyType(property)) {
+    case QVariant::Color:
+        // At the moment it is only possible to reset color values
+        mVariantManager->setValue(property, QColor());
+        break;
+
+    default:
+        qWarning() << "Resetting of property type not supported right now";
+    }
+}
+
 void PropertyBrowser::addMapProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Map"));
 
     QtVariantProperty *orientationProperty =
-            createProperty(OrientationProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Orientation"),
-                           groupProperty);
+            addProperty(OrientationProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Orientation"),
+                        groupProperty);
 
     orientationProperty->setAttribute(QLatin1String("enumNames"), mOrientationNames);
 
-    createProperty(WidthProperty, QVariant::Int, tr("Width"), groupProperty)->setEnabled(false);
-    createProperty(HeightProperty, QVariant::Int, tr("Height"), groupProperty)->setEnabled(false);
-    createProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), groupProperty);
-    createProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), groupProperty);
+    addProperty(WidthProperty, QVariant::Int, tr("Width"), groupProperty)->setEnabled(false);
+    addProperty(HeightProperty, QVariant::Int, tr("Height"), groupProperty)->setEnabled(false);
+    addProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), groupProperty);
+    addProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), groupProperty);
 
-    createProperty(HexSideLengthProperty, QVariant::Int, tr("Tile Side Length (Hex)"), groupProperty);
+    addProperty(HexSideLengthProperty, QVariant::Int, tr("Tile Side Length (Hex)"), groupProperty);
 
     QtVariantProperty *staggerAxisProperty =
-            createProperty(StaggerAxisProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Stagger Axis"),
-                           groupProperty);
+            addProperty(StaggerAxisProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Stagger Axis"),
+                        groupProperty);
 
     staggerAxisProperty->setAttribute(QLatin1String("enumNames"), mStaggerAxisNames);
 
     QtVariantProperty *staggerIndexProperty =
-            createProperty(StaggerIndexProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Stagger Index"),
-                           groupProperty);
+            addProperty(StaggerIndexProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Stagger Index"),
+                        groupProperty);
 
     staggerIndexProperty->setAttribute(QLatin1String("enumNames"), mStaggerIndexNames);
 
     QtVariantProperty *layerFormatProperty =
-            createProperty(LayerFormatProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Tile Layer Format"),
-                           groupProperty);
+            addProperty(LayerFormatProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Tile Layer Format"),
+                        groupProperty);
 
     layerFormatProperty->setAttribute(QLatin1String("enumNames"), mLayerFormatNames);
 
     QtVariantProperty *renderOrderProperty =
-            createProperty(RenderOrderProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Tile Render Order"),
-                           groupProperty);
+            addProperty(RenderOrderProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Tile Render Order"),
+                        groupProperty);
 
     renderOrderProperty->setAttribute(QLatin1String("enumNames"), mRenderOrderNames);
 
-    createProperty(ColorProperty, QVariant::Color, tr("Background Color"), groupProperty);
+    addProperty(ColorProperty, QVariant::Color, tr("Background Color"), groupProperty);
     addProperty(groupProperty);
 }
 
@@ -412,25 +474,26 @@ static QStringList objectTypeNames()
 
 void PropertyBrowser::addMapObjectProperties()
 {
+    // DEFAULT MAP OBJECT PROPERTIES
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Object"));
 
-    createProperty(IdProperty, QVariant::Int, tr("ID"), groupProperty)->setEnabled(false);
-    createProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
+    addProperty(IdProperty, QVariant::Int, tr("ID"), groupProperty)->setEnabled(false);
+    addProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
 
     QtVariantProperty *typeProperty =
-            createProperty(TypeProperty, QVariant::String, tr("Type"), groupProperty);
+            addProperty(TypeProperty, QVariant::String, tr("Type"), groupProperty);
     typeProperty->setAttribute(QLatin1String("suggestions"), objectTypeNames());
 
-    createProperty(VisibleProperty, QVariant::Bool, tr("Visible"), groupProperty);
-    createProperty(XProperty, QVariant::Double, tr("X"), groupProperty);
-    createProperty(YProperty, QVariant::Double, tr("Y"), groupProperty);
-    createProperty(WidthProperty, QVariant::Double, tr("Width"), groupProperty);
-    createProperty(HeightProperty, QVariant::Double, tr("Height"), groupProperty);
-    createProperty(RotationProperty, QVariant::Double, tr("Rotation"), groupProperty);
+    addProperty(VisibleProperty, QVariant::Bool, tr("Visible"), groupProperty);
+    addProperty(XProperty, QVariant::Double, tr("X"), groupProperty);
+    addProperty(YProperty, QVariant::Double, tr("Y"), groupProperty);
+    addProperty(WidthProperty, QVariant::Double, tr("Width"), groupProperty);
+    addProperty(HeightProperty, QVariant::Double, tr("Height"), groupProperty);
+    addProperty(RotationProperty, QVariant::Double, tr("Rotation"), groupProperty);
 
     if (!static_cast<const MapObject*>(mObject)->cell().isEmpty()) {
         QtVariantProperty *flippingProperty =
-                createProperty(FlippingProperty, VariantPropertyManager::flagTypeId(),
+                addProperty(FlippingProperty, VariantPropertyManager::flagTypeId(),
                                tr("Flipping"), groupProperty);
 
         flippingProperty->setAttribute(QLatin1String("flagNames"), mFlippingFlagNames);
@@ -441,11 +504,11 @@ void PropertyBrowser::addMapObjectProperties()
 
 void PropertyBrowser::addLayerProperties(QtProperty *parent)
 {
-    createProperty(NameProperty, QVariant::String, tr("Name"), parent);
-    createProperty(VisibleProperty, QVariant::Bool, tr("Visible"), parent);
+    addProperty(NameProperty, QVariant::String, tr("Name"), parent);
+    addProperty(VisibleProperty, QVariant::Bool, tr("Visible"), parent);
 
     QtVariantProperty *opacityProperty =
-            createProperty(OpacityProperty, QVariant::Double, tr("Opacity"), parent);
+            addProperty(OpacityProperty, QVariant::Double, tr("Opacity"), parent);
     opacityProperty->setAttribute(QLatin1String("minimum"), 0.0);
     opacityProperty->setAttribute(QLatin1String("maximum"), 1.0);
     opacityProperty->setAttribute(QLatin1String("singleStep"), 0.1);
@@ -455,8 +518,8 @@ void PropertyBrowser::addTileLayerProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Tile Layer"));
     addLayerProperties(groupProperty);
-    createProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
-    createProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
+    addProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
+    addProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
     addProperty(groupProperty);
 }
 
@@ -464,16 +527,16 @@ void PropertyBrowser::addObjectGroupProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Object Layer"));
     addLayerProperties(groupProperty);
-    createProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
-    createProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
+    addProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
+    addProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
 
-    createProperty(ColorProperty, QVariant::Color, tr("Color"), groupProperty);
+    addProperty(ColorProperty, QVariant::Color, tr("Color"), groupProperty);
 
     QtVariantProperty *drawOrderProperty =
-            createProperty(DrawOrderProperty,
-                           QtVariantPropertyManager::enumTypeId(),
-                           tr("Drawing Order"),
-                           groupProperty);
+            addProperty(DrawOrderProperty,
+                        QtVariantPropertyManager::enumTypeId(),
+                        tr("Drawing Order"),
+                        groupProperty);
 
     drawOrderProperty->setAttribute(QLatin1String("enumNames"), mDrawOrderNames);
 
@@ -485,40 +548,62 @@ void PropertyBrowser::addImageLayerProperties()
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Image Layer"));
     addLayerProperties(groupProperty);
 
-    QtVariantProperty *imageSourceProperty = createProperty(ImageSourceProperty,
-                                                            VariantPropertyManager::filePathTypeId(),
-                                                            tr("Image"), groupProperty);
+    QtVariantProperty *imageSourceProperty = addProperty(ImageSourceProperty,
+                                                         filePathTypeId(),
+                                                         tr("Image"), groupProperty);
 
     imageSourceProperty->setAttribute(QLatin1String("filter"),
                                       Utils::readableImageFormatsFilter());
 
-    createProperty(ColorProperty, QVariant::Color, tr("Transparent Color"), groupProperty);
-    createProperty(XProperty, QVariant::Int, tr("X"), groupProperty);
-    createProperty(YProperty, QVariant::Int, tr("Y"), groupProperty);
+    addProperty(ColorProperty, QVariant::Color, tr("Transparent Color"), groupProperty);
+    addProperty(OffsetXProperty, QVariant::Double, tr("Horizontal Offset"), groupProperty);
+    addProperty(OffsetYProperty, QVariant::Double, tr("Vertical Offset"), groupProperty);
     addProperty(groupProperty);
 }
 
 void PropertyBrowser::addTilesetProperties()
 {
+    const Tileset *tileset = static_cast<const Tileset*>(mObject);
+
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Tileset"));
-    createProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
-    createProperty(TileOffsetProperty, QVariant::Point, tr("Drawing Offset"), groupProperty);
+
+    if (tileset->isExternal()) {
+        auto property = addProperty(FileNameProperty, filePathTypeId(), tr("Filename"), groupProperty);
+
+        QString filter = QCoreApplication::translate("MainWindow", "All Files (*)");
+        filter += QLatin1String(";;");
+        filter += TsxTilesetFormat().nameFilter();
+        FormatHelper<TilesetFormat> helper(FileFormat::Read, filter);
+
+        property->setAttribute(QStringLiteral("filter"), helper.filter());
+    }
+
+    addProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
+    addProperty(TileOffsetProperty, QVariant::Point, tr("Drawing Offset"), groupProperty);
+
+    QtVariantProperty *columnsProperty = addProperty(ColumnCountProperty, QVariant::Int, tr("Columns"), groupProperty);
+    columnsProperty->setAttribute(QLatin1String("minimum"), 1);
 
     // Next properties we should add only for non 'Collection of Images' tilesets
-    const Tileset *currentTileset = dynamic_cast<const Tileset*>(mObject);
-    if (!currentTileset->imageSource().isEmpty()) {
-        QtVariantProperty *srcImgProperty =
-                createProperty(SourceImageProperty, QVariant::String, tr("Source Image"), groupProperty);
-        QtVariantProperty *tileWidthProperty = createProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), groupProperty);
-        QtVariantProperty *tileHeightProperty = createProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), groupProperty);
-        QtVariantProperty *marginProperty = createProperty(MarginProperty, QVariant::Int, tr("Margin"), groupProperty);
-        QtVariantProperty *spacingProperty = createProperty(SpacingProperty, QVariant::Int, tr("Spacing"), groupProperty);
-        // Make these properties read-only
-        srcImgProperty->setEnabled(false);
+    if (!tileset->isCollection()) {
+        QtVariantProperty *parametersProperty =
+                addProperty(TilesetImageParametersProperty, VariantPropertyManager::tilesetParametersTypeId(), tr("Image"), groupProperty);
+
+        QtVariantProperty *imageSourceProperty = addProperty(ImageSourceProperty, QVariant::String, tr("Source"), parametersProperty);
+        QtVariantProperty *tileWidthProperty = addProperty(TileWidthProperty, QVariant::Int, tr("Tile Width"), parametersProperty);
+        QtVariantProperty *tileHeightProperty = addProperty(TileHeightProperty, QVariant::Int, tr("Tile Height"), parametersProperty);
+        QtVariantProperty *marginProperty = addProperty(MarginProperty, QVariant::Int, tr("Margin"), parametersProperty);
+        QtVariantProperty *spacingProperty = addProperty(SpacingProperty, QVariant::Int, tr("Spacing"), parametersProperty);
+        QtVariantProperty *colorProperty = addProperty(ColorProperty, QVariant::Color, tr("Transparent Color"), parametersProperty);
+
+        // These properties can't be directly edited. To change the parameters,
+        // the TilesetParametersEdit is used.
+        imageSourceProperty->setEnabled(false);
         tileWidthProperty->setEnabled(false);
         tileHeightProperty->setEnabled(false);
         marginProperty->setEnabled(false);
         spacingProperty->setEnabled(false);
+        colorProperty->setEnabled(false);
     }
     addProperty(groupProperty);
 }
@@ -526,22 +611,34 @@ void PropertyBrowser::addTilesetProperties()
 void PropertyBrowser::addTileProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Tile"));
-    createProperty(IdProperty, QVariant::Int, tr("ID"), groupProperty)->setEnabled(false);
+    addProperty(IdProperty, QVariant::Int, tr("ID"), groupProperty)->setEnabled(false);
+    addProperty(WidthProperty, QVariant::Int, tr("Width"), groupProperty)->setEnabled(false);
+    addProperty(HeightProperty, QVariant::Int, tr("Height"), groupProperty)->setEnabled(false);
 
-    QtVariantProperty *probabilityProperty = createProperty(TileProbabilityProperty,
-                                                            QVariant::Double,
-                                                            tr("Probability"),
-                                                            groupProperty);
+    QtVariantProperty *probabilityProperty = addProperty(TileProbabilityProperty,
+                                                         QVariant::Double,
+                                                         tr("Probability"),
+                                                         groupProperty);
     probabilityProperty->setAttribute(QLatin1String("decimals"), 3);
     probabilityProperty->setToolTip(tr("Relative chance this tile will be picked"));
 
+    const Tile *tile = static_cast<const Tile*>(mObject);
+    if (!tile->imageSource().isEmpty()) {
+        QtVariantProperty *imageSourceProperty = addProperty(ImageSourceProperty,
+                                                             filePathTypeId(),
+                                                             tr("Image"), groupProperty);
+
+        imageSourceProperty->setAttribute(QLatin1String("filter"),
+                                          Utils::readableImageFormatsFilter());
+    }
+
     addProperty(groupProperty);
 }
 
 void PropertyBrowser::addTerrainProperties()
 {
     QtProperty *groupProperty = mGroupManager->addProperty(tr("Terrain"));
-    createProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
+    addProperty(NameProperty, QVariant::String, tr("Name"), groupProperty);
     addProperty(groupProperty);
 }
 
@@ -742,10 +839,7 @@ void PropertyBrowser::applyObjectGroupValue(PropertyId id, const QVariant &val)
 
     switch (id) {
     case ColorProperty: {
-        QColor color = val.value<QColor>();
-        if (color == Qt::gray)
-            color = QColor();
-
+        const QColor color = val.value<QColor>();
         command = new ChangeObjectGroupProperties(mMapDocument,
                                                   objectGroup,
                                                   color,
@@ -775,19 +869,16 @@ void PropertyBrowser::applyImageLayerValue(PropertyId id, const QVariant &val)
 
     switch (id) {
     case ImageSourceProperty: {
-        const QString imageSource = val.toString();
+        const FilePath imageSource = val.value<FilePath>();
         const QColor &color = imageLayer->transparentColor();
         undoStack->push(new ChangeImageLayerProperties(mMapDocument,
                                                        imageLayer,
                                                        color,
-                                                       imageSource));
+                                                       imageSource.absolutePath));
         break;
     }
     case ColorProperty: {
-        QColor color = val.value<QColor>();
-        if (color == Qt::gray)
-            color = QColor();
-
+        const QColor color = val.value<QColor>();
         const QString &imageSource = imageLayer->imageSource();
         undoStack->push(new ChangeImageLayerProperties(mMapDocument,
                                                        imageLayer,
@@ -795,20 +886,6 @@ void PropertyBrowser::applyImageLayerValue(PropertyId id, const QVariant &val)
                                                        imageSource));
         break;
     }
-    case XProperty: {
-        QPoint pos(val.toReal(), imageLayer->y());
-        undoStack->push(new ChangeImageLayerPosition(mMapDocument,
-                                                     imageLayer,
-                                                     pos));
-        break;
-    }
-    case YProperty: {
-        QPoint pos(imageLayer->x(), val.toReal());
-        undoStack->push(new ChangeImageLayerPosition(mMapDocument,
-                                                     imageLayer,
-                                                     pos));
-        break;
-    }
     default:
         break;
     }
@@ -820,6 +897,21 @@ void PropertyBrowser::applyTilesetValue(PropertyBrowser::PropertyId id, const QV
     QUndoStack *undoStack = mMapDocument->undoStack();
 
     switch (id) {
+    case FileNameProperty: {
+        FilePath filePath = val.value<FilePath>();
+        QString error;
+        SharedTileset newTileset = Tiled::readTileset(filePath.absolutePath, &error);
+        if (!newTileset) {
+            QMessageBox::critical(window(), tr("Error Reading Tileset"), error);
+            return;
+        }
+
+        int index = mMapDocument->map()->tilesets().indexOf(tileset->sharedPointer());
+        if (index != -1)
+            undoStack->push(new ReplaceTileset(mMapDocument, index, newTileset));
+
+        break;
+    }
     case NameProperty:
         undoStack->push(new RenameTileset(mMapDocument,
                                           tileset,
@@ -830,6 +922,11 @@ void PropertyBrowser::applyTilesetValue(PropertyBrowser::PropertyId id, const QV
                                                     tileset,
                                                     val.toPoint()));
         break;
+    case ColumnCountProperty:
+        undoStack->push(new ChangeTilesetColumnCount(mMapDocument,
+                                                     *tileset,
+                                                     val.toInt()));
+        break;
     default:
         break;
     }
@@ -838,11 +935,22 @@ void PropertyBrowser::applyTilesetValue(PropertyBrowser::PropertyId id, const QV
 void PropertyBrowser::applyTileValue(PropertyId id, const QVariant &val)
 {
     Tile *tile = static_cast<Tile*>(mObject);
+    QUndoStack *undoStack = mMapDocument->undoStack();
 
-    if (id == TileProbabilityProperty) {
-        QUndoStack *undoStack = mMapDocument->undoStack();
+    switch (id) {
+    case TileProbabilityProperty:
         undoStack->push(new ChangeTileProbability(mMapDocument,
-                                                  tile, val.toFloat()));
+                                                  mMapDocument->selectedTiles(),
+                                                  val.toFloat()));
+        break;
+    case ImageSourceProperty: {
+        const FilePath filePath = val.value<FilePath>();
+        undoStack->push(new ChangeTileImageSource(mMapDocument,
+                                                  tile, filePath.absolutePath));
+        break;
+    }
+    default:
+        break;
     }
 }
 
@@ -859,27 +967,53 @@ void PropertyBrowser::applyTerrainValue(PropertyId id, const QVariant &val)
     }
 }
 
+/**
+ * @warning This function does not add the property to the view.
+ */
 QtVariantProperty *PropertyBrowser::createProperty(PropertyId id, int type,
-                                                   const QString &name,
-                                                   QtProperty *parent)
+                                                   const QString &name)
 {
     QtVariantProperty *property = mVariantManager->addProperty(type, name);
+    if (!property) {
+        // fall back to string property for unsupported property types
+        property = mVariantManager->addProperty(QVariant::String, name);
+    }
+
     if (type == QVariant::Bool)
         property->setAttribute(QLatin1String("textVisible"), false);
+    if (type == QVariant::String && id == CustomProperty)
+        property->setAttribute(QLatin1String("multiline"), true);
 
-    parent->addSubProperty(property);
     mPropertyToId.insert(property, id);
 
     if (id != CustomProperty) {
         Q_ASSERT(!mIdToProperty.contains(id));
         mIdToProperty.insert(id, property);
     } else {
+        Q_ASSERT(!mNameToProperty.contains(name));
         mNameToProperty.insert(name, property);
     }
 
     return property;
 }
 
+QtVariantProperty *PropertyBrowser::addProperty(PropertyId id, int type,
+                                                const QString &name,
+                                                QtProperty *parent)
+{
+    QtVariantProperty *property = createProperty(id, type, name);
+
+    parent->addSubProperty(property);
+
+    if (id == CustomProperty) {
+        // Collapse custom color properties, to save space
+        if (type == QVariant::Color)
+            setExpanded(items(property).first(), false);
+    }
+
+    return property;
+}
+
 void PropertyBrowser::addProperties()
 {
     if (!mObject)
@@ -929,6 +1063,8 @@ void PropertyBrowser::removeProperties()
 
 void PropertyBrowser::updateProperties()
 {
+    Q_ASSERT(mObject);
+
     mUpdating = true;
 
     switch (mObject->typeId()) {
@@ -944,10 +1080,7 @@ void PropertyBrowser::updateProperties()
         mIdToProperty[StaggerIndexProperty]->setValue(map->staggerIndex());
         mIdToProperty[LayerFormatProperty]->setValue(map->layerDataFormat());
         mIdToProperty[RenderOrderProperty]->setValue(map->renderOrder());
-        QColor backgroundColor = map->backgroundColor();
-        if (!backgroundColor.isValid())
-            backgroundColor = Qt::darkGray;
-        mIdToProperty[ColorProperty]->setValue(backgroundColor);
+        mIdToProperty[ColorProperty]->setValue(map->backgroundColor());
         break;
     }
     case Object::MapObjectType: {
@@ -978,50 +1111,66 @@ void PropertyBrowser::updateProperties()
         mIdToProperty[NameProperty]->setValue(layer->name());
         mIdToProperty[VisibleProperty]->setValue(layer->isVisible());
         mIdToProperty[OpacityProperty]->setValue(layer->opacity());
+        mIdToProperty[OffsetXProperty]->setValue(layer->offset().x());
+        mIdToProperty[OffsetYProperty]->setValue(layer->offset().y());
 
         switch (layer->layerType()) {
         case Layer::TileLayerType:
-            mIdToProperty[OffsetXProperty]->setValue(layer->offset().x());
-            mIdToProperty[OffsetYProperty]->setValue(layer->offset().y());
             break;
         case Layer::ObjectGroupType: {
-            mIdToProperty[OffsetXProperty]->setValue(layer->offset().x());
-            mIdToProperty[OffsetYProperty]->setValue(layer->offset().y());
             const ObjectGroup *objectGroup = static_cast<const ObjectGroup*>(layer);
-            QColor color = objectGroup->color();
-            if (!color.isValid())
-                color = Qt::gray;
+            const QColor color = objectGroup->color();
             mIdToProperty[ColorProperty]->setValue(color);
             mIdToProperty[DrawOrderProperty]->setValue(objectGroup->drawOrder());
             break;
         }
         case Layer::ImageLayerType:
             const ImageLayer *imageLayer = static_cast<const ImageLayer*>(layer);
-            mIdToProperty[ImageSourceProperty]->setValue(imageLayer->imageSource());
+            mIdToProperty[ImageSourceProperty]->setValue(QVariant::fromValue(FilePath { imageLayer->imageSource() }));
             mIdToProperty[ColorProperty]->setValue(imageLayer->transparentColor());
-            mIdToProperty[XProperty]->setValue(imageLayer->x());
-            mIdToProperty[YProperty]->setValue(imageLayer->y());
             break;
         }
         break;
     }
     case Object::TilesetType: {
-        const Tileset *tileset = static_cast<const Tileset*>(mObject);
+        Tileset *tileset = static_cast<Tileset*>(mObject);
+        const bool external = tileset->isExternal();
+
+        if (QtVariantProperty *fileNameProperty = mIdToProperty.value(FileNameProperty))
+            fileNameProperty->setValue(QVariant::fromValue(FilePath { tileset->fileName() }));
+
         mIdToProperty[NameProperty]->setValue(tileset->name());
         mIdToProperty[TileOffsetProperty]->setValue(tileset->tileOffset());
-        if (!tileset->imageSource().isEmpty()) {
-            mIdToProperty[SourceImageProperty]->setValue(tileset->imageSource());
+        mIdToProperty[ColumnCountProperty]->setValue(tileset->columnCount());
+
+        mIdToProperty[NameProperty]->setEnabled(!external);
+        mIdToProperty[TileOffsetProperty]->setEnabled(!external);
+        mIdToProperty[ColumnCountProperty]->setEnabled(!external && tileset->isCollection());
+
+        if (!tileset->isCollection()) {
+            EmbeddedTileset embeddedTileset(mMapDocument, tileset);
+
+            mIdToProperty[TilesetImageParametersProperty]->setValue(QVariant::fromValue(embeddedTileset));
+            mIdToProperty[ImageSourceProperty]->setValue(QVariant::fromValue(FilePath { tileset->imageSource() }));
             mIdToProperty[TileWidthProperty]->setValue(tileset->tileWidth());
             mIdToProperty[TileHeightProperty]->setValue(tileset->tileHeight());
             mIdToProperty[MarginProperty]->setValue(tileset->margin());
             mIdToProperty[SpacingProperty]->setValue(tileset->tileSpacing());
+            mIdToProperty[ColorProperty]->setValue(tileset->transparentColor());
+
+            mIdToProperty[TilesetImageParametersProperty]->setEnabled(!external);
         }
         break;
     }
     case Object::TileType: {
         const Tile *tile = static_cast<const Tile*>(mObject);
+        const QSize tileSize = tile->size();
         mIdToProperty[IdProperty]->setValue(tile->id());
+        mIdToProperty[WidthProperty]->setValue(tileSize.width());
+        mIdToProperty[HeightProperty]->setValue(tileSize.height());
         mIdToProperty[TileProbabilityProperty]->setValue(tile->probability());
+        if (QtVariantProperty *imageSourceProperty = mIdToProperty.value(ImageSourceProperty))
+            imageSourceProperty->setValue(tile->imageSource());
         break;
     }
     case Object::TerrainType: {
@@ -1039,6 +1188,7 @@ void PropertyBrowser::updateCustomProperties()
     if (!mObject)
         return;
 
+    bool wasUpdating = mUpdating;
     mUpdating = true;
 
     qDeleteAll(mNameToProperty);
@@ -1050,28 +1200,45 @@ void PropertyBrowser::updateCustomProperties()
         if (obj == mObject)
             continue;
 
-        QMapIterator<QString,QString> it(obj->properties());
+        QMapIterator<QString,QVariant> it(obj->properties());
 
         while (it.hasNext()) {
             it.next();
             if (!mCombinedProperties.contains(it.key()))
-                mCombinedProperties.insert(it.key(), tr(""));
+                mCombinedProperties.insert(it.key(), QString());
         }
     }
 
-    QMapIterator<QString,QString> it(mCombinedProperties);
+    // Add properties based on object type, if defined
+    if (mObject->typeId() == Object::MapObjectType) {
+        const QString currentType = static_cast<MapObject*>(mObject)->type();
+        const ObjectTypes objectTypes = Preferences::instance()->objectTypes();
+        for (const ObjectType &type : objectTypes) {
+            if (type.name == currentType) {
+                QMapIterator<QString,QVariant> it(type.defaultProperties);
+                while (it.hasNext()) {
+                    it.next();
+                    if (!mCombinedProperties.contains(it.key()))
+                        mCombinedProperties.insert(it.key(), it.value());
+                }
+            }
+        }
+    }
+
+    QMapIterator<QString,QVariant> it(mCombinedProperties);
 
     while (it.hasNext()) {
         it.next();
-        QtVariantProperty *property = createProperty(CustomProperty,
-                                                     QVariant::String,
-                                                     it.key(),
-                                                     mCustomPropertiesGroup);
+        QtVariantProperty *property = addProperty(CustomProperty,
+                                                  it.value().userType(),
+                                                  it.key(),
+                                                  mCustomPropertiesGroup);
+
         property->setValue(it.value());
         updatePropertyColor(it.key());
     }
 
-    mUpdating = false;
+    mUpdating = wasUpdating;
 }
 
 // If there are other objects selected check if their properties are equal. If not give them a gray color.
@@ -1084,27 +1251,32 @@ void PropertyBrowser::updatePropertyColor(const QString &name)
     QString propertyName = property->propertyName();
     QString propertyValue = property->valueText();
 
+    const auto &objects = mMapDocument->currentObjects();
+
+    QColor textColor = palette().color(QPalette::Active, QPalette::WindowText);
+    QColor disabledTextColor = palette().color(QPalette::Disabled, QPalette::WindowText);
+
     // If one of the objects doesn't have this property then gray out the name and value.
-    for (Object *obj : mMapDocument->currentObjects()) {
+    for (Object *obj : objects) {
         if (!obj->hasProperty(propertyName)) {
-            property->setNameColor(Qt::gray);
-            property->setValueColor(Qt::gray);
+            property->setNameColor(disabledTextColor);
+            property->setValueColor(disabledTextColor);
             return;
         }
     }
 
     // If one of the objects doesn't have the same property value then gray out the value.
-    for (Object *obj : mMapDocument->currentObjects()) {
+    for (Object *obj : objects) {
         if (obj == mObject)
             continue;
         if (obj->property(propertyName) != propertyValue) {
-            property->setValueColor(Qt::gray);
+            property->setValueColor(disabledTextColor);
             return;
         }
     }
 
-    property->setNameColor(Qt::black);
-    property->setValueColor(Qt::black);
+    property->setNameColor(textColor);
+    property->setValueColor(textColor);
 }
 
 } // namespace Internal
