@@ -86,6 +86,7 @@ protected:
 private slots:
     void mapChanged();
     void objectsChanged(const QList<MapObject*> &objects);
+    void objectsTypeChanged(const QList<MapObject*> &objects);
     void layerChanged(int index);
     void objectGroupChanged(ObjectGroup *objectGroup);
     void imageLayerChanged(ImageLayer *imageLayer);
@@ -100,8 +101,12 @@ private slots:
     void selectedObjectsChanged();
     void selectedTilesChanged();
 
+    void objectTypesChanged();
+
     void valueChanged(QtProperty *property, const QVariant &val);
 
+    void resetProperty(QtProperty *property);
+
 private:
     enum PropertyId {
         NameProperty,
@@ -125,13 +130,15 @@ private:
         RenderOrderProperty,
         LayerFormatProperty,
         ImageSourceProperty,
+        TilesetImageParametersProperty,
         FlippingProperty,
         DrawOrderProperty,
+        FileNameProperty,
         TileOffsetProperty,
-        SourceImageProperty,
         MarginProperty,
         SpacingProperty,
         TileProbabilityProperty,
+        ColumnCountProperty,
         IdProperty,
         CustomProperty
     };
@@ -159,8 +166,13 @@ private:
 
     QtVariantProperty *createProperty(PropertyId id,
                                       int type,
-                                      const QString &name,
-                                      QtProperty *parent);
+                                      const QString &name);
+
+    using QtTreePropertyBrowser::addProperty;
+    QtVariantProperty *addProperty(PropertyId id,
+                                   int type,
+                                   const QString &name,
+                                   QtProperty *parent);
 
     void addProperties();
     void removeProperties();
