@@ -44,7 +44,10 @@ public:
     const QSize &newSize() const;
     const QPoint &offset() const;
 
+    bool removeObjects() const;
+
 private slots:
+    void removeObjectsToggled(bool removeObjects);
     void updateOffsetBounds(const QRect &bounds);
 
 private:
