@@ -23,7 +23,6 @@
 #include "addremovemapobject.h"
 #include "editpolygontool.h"
 #include "changetileobjectgroup.h"
-#include "clipboardmanager.h"
 #include "createobjecttool.h"
 #include "createrectangleobjecttool.h"
 #include "createellipseobjecttool.h"
@@ -38,6 +37,7 @@
 #include "mapview.h"
 #include "objectgroup.h"
 #include "objectselectiontool.h"
+#include "propertiesdock.h"
 #include "tile.h"
 #include "tilelayer.h"
 #include "tileset.h"
@@ -46,6 +46,7 @@
 #include "zoomable.h"
 
 #include <QCloseEvent>
+#include <QCoreApplication>
 #include <QComboBox>
 #include <QShortcut>
 #include <QStatusBar>
@@ -63,6 +64,7 @@ TileCollisionEditor::TileCollisionEditor(QWidget *parent)
     , mMapScene(new MapScene(this))
     , mMapView(new MapView(this, MapView::NoStaticContents))
     , mToolManager(new ToolManager(this))
+    , mPropertiesDock(new PropertiesDock(this))
     , mApplyingChanges(false)
     , mSynchronizing(false)
 {
@@ -73,6 +75,15 @@ TileCollisionEditor::TileCollisionEditor(QWidget *parent)
     layout->setSpacing(0);
     layout->setMargin(5);
 
+    // We re-use the PropertiesDock class in order to manipulate properties on
+    // collision objects. This instance of the PropertiesDock is best left with
+    // the docking features disabled though as it only belongs to the collision
+    // editor.
+    QDockWidget::DockWidgetFeatures features = QDockWidget::NoDockWidgetFeatures;
+    mPropertiesDock->setFeatures(features);
+    mPropertiesDock->setContextMenuPolicy(Qt::PreventContextMenu);
+    addDockWidget(Qt::LeftDockWidgetArea, mPropertiesDock);
+
     mMapView->setScene(mMapScene);
 
     mMapView->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
@@ -84,6 +95,7 @@ TileCollisionEditor::TileCollisionEditor(QWidget *parent)
     CreateObjectTool *polylineObjectsTool = new CreatePolylineObjectTool(this);;
 
     QToolBar *toolBar = new QToolBar(this);
+    toolBar->setObjectName(QLatin1String("TileCollisionEditorToolBar"));
     toolBar->setMovable(false);
     toolBar->setFloatable(false);
     toolBar->setContextMenuPolicy(Qt::ActionsContextMenu);
@@ -114,6 +126,7 @@ TileCollisionEditor::TileCollisionEditor(QWidget *parent)
     QShortcut *cutShortcut = new QShortcut(QKeySequence::Cut, this);
     QShortcut *copyShortcut = new QShortcut(QKeySequence::Copy, this);
     QShortcut *pasteShortcut = new QShortcut(QKeySequence::Paste, this);
+    QShortcut *pasteInPlaceShortcut = new QShortcut(QCoreApplication::translate("MainWindow", "Ctrl+Shift+V"), this);
     QShortcut *deleteShortcut = new QShortcut(QKeySequence::Delete, this);
     QShortcut *deleteShortcut2 = new QShortcut(QKeySequence(Qt::Key_Backspace), this);
 
@@ -122,6 +135,7 @@ TileCollisionEditor::TileCollisionEditor(QWidget *parent)
     connect(cutShortcut, SIGNAL(activated()), SLOT(cut()));
     connect(copyShortcut, SIGNAL(activated()), SLOT(copy()));
     connect(pasteShortcut, SIGNAL(activated()), SLOT(paste()));
+    connect(pasteInPlaceShortcut, SIGNAL(activated()), SLOT(pasteInPlace()));
     connect(deleteShortcut, SIGNAL(activated()), SLOT(delete_()));
     connect(deleteShortcut2, SIGNAL(activated()), SLOT(delete_()));
 
@@ -169,7 +183,9 @@ void TileCollisionEditor::setTile(Tile *tile)
     MapDocument *previousDocument = mMapScene->mapDocument();
 
     if (tile) {
-        mMapView->setEnabled(!mTile->tileset()->isExternal());
+        bool isExternal = mTile->tileset()->isExternal();
+        mMapView->setEnabled(!isExternal);
+        mPropertiesDock->setEnabled(!isExternal);
 
         Map *map = new Map(Map::Orthogonal, 1, 1, tile->width(), tile->height());
         map->addTileset(tile->sharedTileset());
@@ -185,26 +201,39 @@ void TileCollisionEditor::setTile(Tile *tile)
             objectGroup = new ObjectGroup;
 
         objectGroup->setDrawOrder(ObjectGroup::IndexOrder);
+        map->setNextObjectId(objectGroup->highestObjectId() + 1);
         map->addLayer(objectGroup);
 
         MapDocument *mapDocument = new MapDocument(map);
         mMapScene->setMapDocument(mapDocument);
 
         mToolManager->setMapDocument(mapDocument);
+        mPropertiesDock->setMapDocument(mapDocument);
+
         mapDocument->setCurrentLayerIndex(1);
 
         mMapScene->enableSelectedTool();
 
-        connect(mapDocument->undoStack(), SIGNAL(indexChanged(int)),
-                SLOT(applyChanges()));
+        connect(mapDocument->undoStack(), &QUndoStack::indexChanged,
+                this, &TileCollisionEditor::applyChanges);
+
+        connect(mapDocument, &MapDocument::selectedObjectsChanged,
+                this, &TileCollisionEditor::selectedObjectsChanged);
+
     } else {
         mMapView->setEnabled(false);
+        mPropertiesDock->setEnabled(false);
         mMapScene->setMapDocument(nullptr);
         mToolManager->setMapDocument(nullptr);
+        mPropertiesDock->setMapDocument(nullptr);
     }
 
     if (previousDocument) {
-        previousDocument->undoStack()->disconnect(this);
+        // Explicitly disconnect early from this signal, since it can get fired
+        // from the QUndoStack destructor.
+        disconnect(previousDocument->undoStack(), &QUndoStack::indexChanged,
+                   this, &TileCollisionEditor::applyChanges);
+
         delete previousDocument;
     }
 }
@@ -269,8 +298,10 @@ void TileCollisionEditor::tileObjectGroupChanged(Tile *tile)
 
 void TileCollisionEditor::tilesetFileNameChanged(Tileset *tileset)
 {
-    if (mTile && mTile->tileset() == tileset)
+    if (mTile && mTile->tileset() == tileset) {
         mMapView->setEnabled(!tileset->isExternal());
+        mPropertiesDock->setEnabled(!tileset->isExternal());
+    }
 }
 
 void TileCollisionEditor::currentObjectChanged(Object *object)
@@ -314,6 +345,16 @@ void TileCollisionEditor::copy()
 }
 
 void TileCollisionEditor::paste()
+{
+    paste(ClipboardManager::PasteDefault);
+}
+
+void TileCollisionEditor::pasteInPlace()
+{
+    paste(ClipboardManager::PasteInPlace);
+}
+
+void TileCollisionEditor::paste(ClipboardManager::PasteFlags flags)
 {
     if (!mTile)
         return;
@@ -333,7 +374,7 @@ void TileCollisionEditor::paste()
         MapDocument *dummyDocument = mMapScene->mapDocument();
         clipboardManager->pasteObjectGroup(objectGroup,
                                            dummyDocument, mMapView,
-                                           ClipboardManager::NoTileObjects);
+                                           flags | ClipboardManager::PasteNoTileObjects);
     }
 }
 
@@ -350,12 +391,19 @@ void TileCollisionEditor::delete_(Operation operation)
     QUndoStack *undoStack = dummyDocument->undoStack();
     undoStack->beginMacro(operation == Delete ? tr("Delete") : tr("Cut"));
 
-    foreach (MapObject *mapObject, selectedObjects)
+    for (MapObject *mapObject : selectedObjects)
         undoStack->push(new RemoveMapObject(dummyDocument, mapObject));
 
     undoStack->endMacro();
 }
 
+void TileCollisionEditor::selectedObjectsChanged()
+{
+    MapDocument *dummyDocument = mMapScene->mapDocument();
+    if (dummyDocument->selectedObjects().isEmpty())
+        dummyDocument->setCurrentObject(dummyDocument->map()->layerAt(1));
+}
+
 void TileCollisionEditor::changeEvent(QEvent *e)
 {
     QMainWindow::changeEvent(e);
