@@ -21,6 +21,8 @@
 #ifndef TILECOLLISIONEDITOR_H
 #define TILECOLLISIONEDITOR_H
 
+#include "clipboardmanager.h"
+
 #include <QMainWindow>
 
 namespace Tiled {
@@ -35,6 +37,7 @@ class AbstractTool;
 class MapDocument;
 class MapScene;
 class MapView;
+class PropertiesDock;
 class ToolManager;
 
 class TileCollisionEditor : public QMainWindow
@@ -76,8 +79,12 @@ private slots:
     void cut();
     void copy();
     void paste();
+    void pasteInPlace();
+    void paste(ClipboardManager::PasteFlags flags);
     void delete_(Operation operation = Delete);
 
+    void selectedObjectsChanged();
+
 private:
     void retranslateUi();
 
@@ -86,6 +93,7 @@ private:
     MapScene *mMapScene;
     MapView *mMapView;
     ToolManager *mToolManager;
+    PropertiesDock *mPropertiesDock;
     bool mApplyingChanges;
     bool mSynchronizing;
 };
