@@ -24,7 +24,9 @@
 
 #include "undocommands.h"
 
+#include <QColor>
 #include <QPoint>
+#include <QSize>
 #include <QUndoCommand>
 
 namespace Tiled {
@@ -72,6 +74,61 @@ private:
     QPoint mNewTileOffset;
 };
 
+struct TilesetParameters
+{
+    TilesetParameters()
+        : tileSpacing(0)
+        , margin(0)
+    {}
+
+    explicit TilesetParameters(const Tileset &tileset);
+
+    bool operator != (const TilesetParameters &other) const;
+
+    QString imageSource;
+    QColor transparentColor;
+    QSize tileSize;
+    int tileSpacing;
+    int margin;
+};
+
+class ChangeTilesetParameters : public QUndoCommand
+{
+public:
+    ChangeTilesetParameters(MapDocument *mapDocument,
+                            Tileset &tileset,
+                            const TilesetParameters &parameters);
+
+    void undo() override;
+    void redo() override;
+
+private:
+    void apply(const TilesetParameters &parameters);
+
+    MapDocument *mMapDocument;
+    Tileset &mTileset;
+    TilesetParameters mOldParameters;
+    TilesetParameters mNewParameters;
+};
+
+class ChangeTilesetColumnCount : public QUndoCommand
+{
+public:
+    ChangeTilesetColumnCount(MapDocument *mapDocument,
+                             Tileset &tileset,
+                             int columnCount);
+
+    void undo() override { swap(); }
+    void redo() override { swap(); }
+
+private:
+    void swap();
+
+    MapDocument *mMapDocument;
+    Tileset &mTileset;
+    int mColumnCount;
+};
+
 } // namespace Internal
 } // namespace Tiled
 
