@@ -43,11 +43,11 @@ TilesetManager::TilesetManager():
     mChangedFilesTimer.setInterval(500);
     mChangedFilesTimer.setSingleShot(true);
 
-    connect(&mChangedFilesTimer, SIGNAL(timeout()),
-            this, SLOT(fileChangedTimeout()));
+    connect(&mChangedFilesTimer, &QTimer::timeout,
+            this, &TilesetManager::fileChangedTimeout);
 
-    connect(mAnimationDriver, SIGNAL(update(int)),
-            this, SLOT(advanceTileAnimations(int)));
+    connect(mAnimationDriver, &TileAnimationDriver::update,
+            this, &TilesetManager::advanceTileAnimations);
 }
 
 TilesetManager::~TilesetManager()
@@ -73,29 +73,15 @@ void TilesetManager::deleteInstance()
 
 SharedTileset TilesetManager::findTileset(const QString &fileName) const
 {
-    foreach (const SharedTileset &tileset, tilesets())
+    const QList<SharedTileset> &_tilesets = tilesets();
+
+    for (const SharedTileset &tileset : _tilesets)
         if (tileset->fileName() == fileName)
             return tileset;
 
     return SharedTileset();
 }
 
-SharedTileset TilesetManager::findTileset(const TilesetSpec &spec) const
-{
-    foreach (const SharedTileset &tileset, tilesets()) {
-        if (tileset->imageSource() == spec.imageSource
-            && tileset->tileWidth() == spec.tileWidth
-            && tileset->tileHeight() == spec.tileHeight
-            && tileset->tileSpacing() == spec.tileSpacing
-            && tileset->margin() == spec.margin)
-        {
-            return tileset;
-        }
-    }
-
-    return SharedTileset();
-}
-
 void TilesetManager::addReference(const SharedTileset &tileset)
 {
     if (mTilesets.contains(tileset)) {
@@ -121,13 +107,13 @@ void TilesetManager::removeReference(const SharedTileset &tileset)
 
 void TilesetManager::addReferences(const QVector<SharedTileset> &tilesets)
 {
-    foreach (const SharedTileset &tileset, tilesets)
+    for (const SharedTileset &tileset : tilesets)
         addReference(tileset);
 }
 
 void TilesetManager::removeReferences(const QVector<SharedTileset> &tilesets)
 {
-    foreach (const SharedTileset &tileset, tilesets)
+    for (const SharedTileset &tileset : tilesets)
         removeReference(tileset);
 }
 
@@ -141,8 +127,7 @@ void TilesetManager::forceTilesetReload(SharedTileset &tileset)
     if (!mTilesets.contains(tileset))
         return;
 
-    QString fileName = tileset->imageSource();
-    if (tileset->loadFromImage(fileName))
+    if (tileset->loadImage())
         emit tilesetChanged(tileset.data());
 }
 
@@ -152,6 +137,9 @@ void TilesetManager::setReloadTilesetsOnChange(bool enabled)
     // TODO: Clear the file system watcher when disabled
 }
 
+/**
+ * Sets whether tile animations are running.
+ */
 void TilesetManager::setAnimateTiles(bool enabled)
 {
     // TODO: Avoid running the driver when there are no animated tiles
@@ -166,6 +154,17 @@ bool TilesetManager::animateTiles() const
     return mAnimationDriver->state() == QAbstractAnimation::Running;
 }
 
+void TilesetManager::tilesetImageSourceChanged(const Tileset &tileset,
+                                               const QString &oldImageSource)
+{
+    Q_ASSERT(mTilesets.contains(tileset.sharedPointer()));
+    Q_ASSERT(!oldImageSource.isEmpty());
+    Q_ASSERT(!tileset.imageSource().isEmpty());
+
+    mWatcher->removePath(oldImageSource);
+    mWatcher->addPath(tileset.imageSource());
+}
+
 void TilesetManager::fileChanged(const QString &path)
 {
     if (!mReloadTilesetsOnChange)
@@ -192,14 +191,37 @@ void TilesetManager::fileChangedTimeout()
     mChangedFiles.clear();
 }
 
-void TilesetManager::advanceTileAnimations(int ms)
+/**
+ * Resets all tile animations. Used to keep animations synchronized when they
+ * are edited.
+ */
+void TilesetManager::resetTileAnimations()
 {
+    const QList<SharedTileset> &_tilesets = tilesets();
+
     // TODO: This could be more optimal by keeping track of the list of
     // actually animated tiles
-    foreach (const SharedTileset &tileset, tilesets()) {
+    for (const SharedTileset &tileset : _tilesets) {
         bool imageChanged = false;
 
-        foreach (Tile *tile, tileset->tiles())
+        for (Tile *tile : tileset->tiles())
+            imageChanged |= tile->resetAnimation();
+
+        if (imageChanged)
+            emit repaintTileset(tileset.data());
+    }
+}
+
+void TilesetManager::advanceTileAnimations(int ms)
+{
+    const QList<SharedTileset> &_tilesets = tilesets();
+
+    // TODO: This could be more optimal by keeping track of the list of
+    // actually animated tiles
+    for (const SharedTileset &tileset : _tilesets) {
+        bool imageChanged = false;
+
+        for (Tile *tile : tileset->tiles())
             imageChanged |= tile->advanceAnimation(ms);
 
         if (imageChanged)
