@@ -37,19 +37,6 @@ namespace Internal {
 class FileSystemWatcher;
 class TileAnimationDriver;
 
-/**
- * A tileset specification that uniquely identifies a certain tileset. Does not
- * take into account tile properties!
- */
-struct TilesetSpec
-{
-    QString imageSource;
-    int tileWidth;
-    int tileHeight;
-    int tileSpacing;
-    int margin;
-};
-
 /**
  * The tileset manager keeps track of all tilesets used by loaded maps. It also
  * watches the tileset images for changes and will attempt to reload them when
@@ -77,12 +64,6 @@ public:
      */
     SharedTileset findTileset(const QString &fileName) const;
 
-    /**
-     * Searches for a tileset matching the given specification.
-     * @return a tileset matching the given specification, or 0 if none exists
-     */
-    SharedTileset findTileset(const TilesetSpec &spec) const;
-
     /**
      * Adds a tileset reference. This will make sure the tileset is watched for
      * changes and can be found using findTileset().
@@ -124,11 +105,12 @@ public:
     void setReloadTilesetsOnChange(bool enabled);
     bool reloadTilesetsOnChange() const;
 
-    /**
-     * Sets whether tile animations are running.
-     */
     void setAnimateTiles(bool enabled);
     bool animateTiles() const;
+    void resetTileAnimations();
+
+    void tilesetImageSourceChanged(const Tileset &tileset,
+                                   const QString &oldImageSource);
 
 signals:
     /**
