@@ -38,6 +38,7 @@
 #include <QMenu>
 #include <QPainter>
 #include <QPinchGesture>
+#include <QScrollBar>
 #include <QUndoCommand>
 #include <QWheelEvent>
 #include <QtCore/qmath.h>
@@ -217,7 +218,19 @@ void TileDelegate::paint(QPainter *painter,
     const QPixmap &tileImage = tile->image();
     const int extra = mTilesetView->drawGrid() ? 1 : 0;
     const qreal zoom = mTilesetView->scale();
-    const QSize tileSize = tileImage.size() * zoom;
+
+    QSize tileSize = tileImage.size();
+    if (tileImage.isNull()) {
+        Tileset *tileset = model->tileset();
+        if (tileset->isCollection()) {
+            tileSize = QSize(32, 32);
+        } else {
+            int max = std::max(tileset->tileWidth(), tileset->tileWidth());
+            int min = std::min(max, 32);
+            tileSize = QSize(min, min);
+        }
+    }
+    tileSize *= zoom;
 
     // Compute rectangle to draw the image in: bottom- and left-aligned
     QRect targetRect = option.rect.adjusted(0, 0, -extra, -extra);
@@ -229,7 +242,11 @@ void TileDelegate::paint(QPainter *painter,
         if (zoomable->smoothTransform())
             painter->setRenderHint(QPainter::SmoothPixmapTransform);
 
-    painter->drawPixmap(targetRect, tileImage);
+    if (!tileImage.isNull())
+        painter->drawPixmap(targetRect, tileImage);
+    else
+        mTilesetView->imageMissingIcon().paint(painter, targetRect, Qt::AlignBottom | Qt::AlignLeft);
+
 
     // Overlay with film strip when animated
     if (mTilesetView->markAnimatedTiles() && tile->isAnimated()) {
@@ -311,9 +328,22 @@ QSize TileDelegate::sizeHint(const QStyleOptionViewItem & /* option */,
     const int extra = mTilesetView->drawGrid() ? 1 : 0;
 
     if (const Tile *tile = m->tileAt(index)) {
-        const QSize tileSize = tile->size() * mTilesetView->scale();
-        return QSize(tileSize.width() + extra,
-                     tileSize.height() + extra);
+        const QPixmap &image = tile->image();
+        QSize tileSize = image.size();
+
+        if (image.isNull()) {
+            Tileset *tileset = m->tileset();
+            if (tileset->isCollection()) {
+                tileSize = QSize(32, 32);
+            } else {
+                int max = std::max(tileset->tileWidth(), tileset->tileWidth());
+                int min = std::min(max, 32);
+                tileSize = QSize(min, min);
+            }
+        }
+
+        return QSize(tileSize.width() * mTilesetView->scale() + extra,
+                     tileSize.height() * mTilesetView->scale() + extra);
     }
 
     return QSize(extra, extra);
@@ -332,6 +362,8 @@ TilesetView::TilesetView(QWidget *parent)
     , mTerrainId(-1)
     , mHoveredCorner(0)
     , mTerrainChanged(false)
+    , mHandScrolling(false)
+    , mImageMissingIcon(QStringLiteral("://images/32x32/image-missing.png"))
 {
     setHorizontalScrollMode(QAbstractItemView::ScrollPerPixel);
     setVerticalScrollMode(QAbstractItemView::ScrollPerPixel);
@@ -376,10 +408,8 @@ int TilesetView::sizeHintForColumn(int column) const
     const TilesetModel *model = tilesetModel();
     if (!model)
         return -1;
-#if QT_VERSION >= 0x050200
-    if (model->tileset()->imageSource().isEmpty())
+    if (model->tileset()->isCollection())
         return QTableView::sizeHintForColumn(column);
-#endif
 
     const int tileWidth = model->tileset()->tileWidth();
     return qRound(tileWidth * scale()) + (mDrawGrid ? 1 : 0);
@@ -391,10 +421,8 @@ int TilesetView::sizeHintForRow(int row) const
     const TilesetModel *model = tilesetModel();
     if (!model)
         return -1;
-#if QT_VERSION >= 0x050200
-    if (model->tileset()->imageSource().isEmpty())
+    if (model->tileset()->isCollection())
         return QTableView::sizeHintForRow(row);
-#endif
 
     const int tileHeight = model->tileset()->tileHeight();
     return qRound(tileHeight * scale()) + (mDrawGrid ? 1 : 0);
@@ -457,8 +485,19 @@ void TilesetView::setTerrainId(int terrainId)
         viewport()->update();
 }
 
+QIcon TilesetView::imageMissingIcon() const
+{
+    return QIcon::fromTheme(QLatin1String("image-missing"), mImageMissingIcon);
+}
+
 void TilesetView::mousePressEvent(QMouseEvent *event)
 {
+    if (event->button() == Qt::MidButton && isActiveWindow()) {
+        mLastMousePos = event->globalPos();
+        setHandScrolling(true);
+        return;
+    }
+
     if (!mEditTerrain) {
         QTableView::mousePressEvent(event);
         return;
@@ -470,6 +509,21 @@ void TilesetView::mousePressEvent(QMouseEvent *event)
 
 void TilesetView::mouseMoveEvent(QMouseEvent *event)
 {
+    if (mHandScrolling) {
+        auto *hBar = horizontalScrollBar();
+        auto *vBar = verticalScrollBar();
+        const QPoint d = event->globalPos() - mLastMousePos;
+
+        int horizontalValue = hBar->value() + (isRightToLeft() ? d.x() : -d.x());
+        int verticalValue = vBar->value() - d.y();
+
+        hBar->setValue(horizontalValue);
+        vBar->setValue(verticalValue);
+
+        mLastMousePos = event->globalPos();
+        return;
+    }
+
     if (!mEditTerrain) {
         QTableView::mouseMoveEvent(event);
         return;
@@ -505,6 +559,11 @@ void TilesetView::mouseMoveEvent(QMouseEvent *event)
 
 void TilesetView::mouseReleaseEvent(QMouseEvent *event)
 {
+    if (event->button() == Qt::MidButton) {
+        setHandScrolling(false);
+        return;
+    }
+
     if (!mEditTerrain) {
         QTableView::mouseReleaseEvent(event);
         return;
@@ -624,13 +683,13 @@ void TilesetView::setDrawGrid(bool drawGrid)
 {
     mDrawGrid = drawGrid;
     if (TilesetModel *model = tilesetModel())
-        model->tilesetChanged();
+        model->resetModel();
 }
 
 void TilesetView::adjustScale()
 {
     if (TilesetModel *model = tilesetModel())
-        model->tilesetChanged();
+        model->resetModel();
 }
 
 void TilesetView::applyTerrain()
@@ -669,3 +728,16 @@ Tile *TilesetView::currentTile() const
     const TilesetModel *model = tilesetModel();
     return model ? model->tileAt(currentIndex()) : nullptr;
 }
+
+void TilesetView::setHandScrolling(bool handScrolling)
+{
+    if (mHandScrolling == handScrolling)
+        return;
+
+    mHandScrolling = handScrolling;
+
+    if (mHandScrolling)
+        setCursor(QCursor(Qt::ClosedHandCursor));
+    else
+        unsetCursor();
+}
