@@ -109,6 +109,8 @@ public:
     QModelIndex hoveredIndex() const { return mHoveredIndex; }
     int hoveredCorner() const { return mHoveredCorner; }
 
+    QIcon imageMissingIcon() const;
+
 signals:
     void createNewTerrain(Tile *tile);
     void terrainImageSelected(Tile *tile);
@@ -134,6 +136,7 @@ private:
     void applyTerrain();
     void finishTerrainChange();
     Tile *currentTile() const;
+    void setHandScrolling(bool handScrolling);
 
     Zoomable *mZoomable;
     MapDocument *mMapDocument;
@@ -146,6 +149,11 @@ private:
     QModelIndex mHoveredIndex;
     int mHoveredCorner;
     bool mTerrainChanged;
+
+    bool mHandScrolling;
+    QPoint mLastMousePos;
+
+    const QIcon mImageMissingIcon;
 };
 
 inline bool TilesetView::markAnimatedTiles() const
