@@ -28,12 +28,15 @@ namespace Tiled {
 namespace Internal {
 
 class FileEdit;
+class TextPropertyEdit;
+class TilesetParametersEdit;
 
 /**
  * Extension of the QtVariantEditorFactory that adds support for a FileEdit,
  * used for editing file references.
  *
- * It also adds support for a "suggestions" attribute for string values.
+ * It also adds support for "suggestions" and "multiline" attributes for string
+ * values.
  */
 class VariantEditorFactory : public QtVariantEditorFactory
 {
@@ -46,6 +49,9 @@ public:
 
     ~VariantEditorFactory();
 
+signals:
+    void resetProperty(QtProperty *property);
+
 protected:
     void connectPropertyManager(QtVariantPropertyManager *manager) override;
     QWidget *createEditor(QtVariantPropertyManager *manager,
@@ -58,12 +64,19 @@ private slots:
     void slotPropertyAttributeChanged(QtProperty *property,
                                       const QString &attribute,
                                       const QVariant &value);
-    void slotSetValue(const QString &value);
+    void fileEditFilePathChanged(const QString &value);
+    void textPropertyEditTextChanged(const QString &value);
     void slotEditorDestroyed(QObject *object);
 
 private:
-    QMap<QtProperty *, QList<FileEdit *> > mCreatedEditors;
-    QMap<FileEdit *, QtProperty *> mEditorToProperty;
+    QMap<QtProperty *, QList<FileEdit *> > mCreatedFileEdits;
+    QMap<FileEdit *, QtProperty *> mFileEditToProperty;
+
+    QMap<QtProperty *, QList<TilesetParametersEdit *> > mCreatedTilesetEdits;
+    QMap<TilesetParametersEdit *, QtProperty *> mTilesetEditToProperty;
+
+    QMap<QtProperty *, QList<TextPropertyEdit *> > mCreatedTextPropertyEdits;
+    QMap<TextPropertyEdit *, QtProperty *> mTextPropertyEditToProperty;
 };
 
 } // namespace Internal
