@@ -21,39 +21,34 @@
 
 #include "variantpropertymanager.h"
 
+#include "mapdocument.h"
+#include "textpropertyedit.h"
+
+#include <QFileInfo>
+
 namespace Tiled {
 namespace Internal {
 
-class FilePathPropertyType {};
+class TilesetParametersPropertyType {};
 
 } // namespace Tiled
 } // namespace Internal
 
 // Needs to be up here rather than at the bottom of the file to make a
 // static_assert in qMetaTypeId work (as of C++11)
-Q_DECLARE_METATYPE(Tiled::Internal::FilePathPropertyType)
+Q_DECLARE_METATYPE(Tiled::Internal::TilesetParametersPropertyType)
 
 
 namespace Tiled {
 namespace Internal {
 
-int VariantPropertyManager::filePathTypeId()
+VariantPropertyManager::VariantPropertyManager(QObject *parent)
+    : QtVariantPropertyManager(parent)
+    , mSuggestionsAttribute(QStringLiteral("suggestions"))
+    , mMultilineAttribute(QStringLiteral("multiline"))
+    , mImageMissingIcon(QStringLiteral("://images/16x16/image-missing.png"))
 {
-    return qMetaTypeId<FilePathPropertyType>();
-}
-
-bool VariantPropertyManager::isPropertyTypeSupported(int propertyType) const
-{
-    if (propertyType == filePathTypeId())
-        return true;
-    return QtVariantPropertyManager::isPropertyTypeSupported(propertyType);
-}
-
-int VariantPropertyManager::valueType(int propertyType) const
-{
-    if (propertyType == filePathTypeId())
-        return QVariant::String;
-    return QtVariantPropertyManager::valueType(propertyType);
+    mImageMissingIcon.addPixmap(QPixmap(QStringLiteral("://images/32x32/image-missing.png")));
 }
 
 QVariant VariantPropertyManager::value(const QtProperty *property) const
@@ -63,12 +58,30 @@ QVariant VariantPropertyManager::value(const QtProperty *property) const
     return QtVariantPropertyManager::value(property);
 }
 
+bool VariantPropertyManager::isPropertyTypeSupported(int propertyType) const
+{
+    if (propertyType == filePathTypeId())
+        return true;
+    if (propertyType == tilesetParametersTypeId())
+        return true;
+    return QtVariantPropertyManager::isPropertyTypeSupported(propertyType);
+}
+
+int VariantPropertyManager::valueType(int propertyType) const
+{
+    if (propertyType == filePathTypeId())
+        return QVariant::String;
+    if (propertyType == tilesetParametersTypeId())
+        return qMetaTypeId<EmbeddedTileset>();
+    return QtVariantPropertyManager::valueType(propertyType);
+}
+
 QStringList VariantPropertyManager::attributes(int propertyType) const
 {
     if (propertyType == filePathTypeId()) {
-        QStringList attr;
-        attr << QLatin1String("filter");
-        return attr;
+        return QStringList {
+            QStringLiteral("filter")
+        };
     }
     return QtVariantPropertyManager::attributes(propertyType);
 }
@@ -92,32 +105,87 @@ QVariant VariantPropertyManager::attributeValue(const QtProperty *property,
             return mValues[property].filter;
         return QVariant();
     }
-    if (attribute == mSuggestionsAttribute && mSuggestions.contains(property))
-        return mSuggestions[property];
+    if (mStringAttributes.contains(property)) {
+        if (attribute == mSuggestionsAttribute)
+            return mStringAttributes[property].suggestions;
+        if (attribute == mMultilineAttribute)
+            return mStringAttributes[property].multiline;
+    }
 
     return QtVariantPropertyManager::attributeValue(property, attribute);
 }
 
+int VariantPropertyManager::tilesetParametersTypeId()
+{
+    return qMetaTypeId<TilesetParametersPropertyType>();
+}
+
 QString VariantPropertyManager::valueText(const QtProperty *property) const
 {
-    if (mValues.contains(property))
-        return mValues[property].value;
+    if (mValues.contains(property)) {
+        QVariant value = mValues[property].value;
+        int typeId = propertyType(property);
+
+        if (typeId == filePathTypeId()) {
+            FilePath filePath = value.value<FilePath>();
+            return QFileInfo(filePath.absolutePath).fileName();
+        }
+
+        if (typeId == tilesetParametersTypeId()) {
+            EmbeddedTileset embeddedTileset = value.value<EmbeddedTileset>();
+            if (embeddedTileset.tileset())
+                return QFileInfo(embeddedTileset.tileset()->imageSource()).fileName();
+        }
+
+        return value.toString();
+    }
+
+    auto stringAttributesIt = mStringAttributes.find(property);
+    if (stringAttributesIt != mStringAttributes.end()) {
+        if ((*stringAttributesIt).multiline)
+            return escapeNewlines(value(property).toString());
+    }
+
     return QtVariantPropertyManager::valueText(property);
 }
 
+QIcon VariantPropertyManager::valueIcon(const QtProperty *property) const
+{
+    if (mValues.contains(property)) {
+        QVariant value = mValues[property].value;
+        QString filePath;
+        int typeId = propertyType(property);
+
+        if (typeId == filePathTypeId())
+            filePath = value.value<FilePath>().absolutePath;
+
+        if (typeId == tilesetParametersTypeId()) {
+            EmbeddedTileset embeddedTileset = value.value<EmbeddedTileset>();
+            if (embeddedTileset.tileset())
+                filePath = embeddedTileset.tileset()->imageSource();
+        }
+
+        // TODO: This assumes the file path is an image reference. It should be
+        // replaced with a more generic icon.
+        if (filePath.isEmpty() || !QFile::exists(filePath))
+            return QIcon::fromTheme(QLatin1String("image-missing"), mImageMissingIcon);
+        else
+            return mIconProvider.icon(QFileInfo(filePath));
+    }
+
+    return QtVariantPropertyManager::valueIcon(property);
+}
+
 void VariantPropertyManager::setValue(QtProperty *property, const QVariant &val)
 {
     if (mValues.contains(property)) {
-        if (val.type() != QVariant::String && !val.canConvert(QVariant::String))
-            return;
-        QString str = val.toString();
         Data d = mValues[property];
-        if (d.value == str)
+        if (d.value == val)
             return;
-        d.value = str;
+        d.value = val;
         mValues[property] = d;
         emit propertyChanged(property);
-        emit valueChanged(property, str);
+        emit valueChanged(property, val);
         return;
     }
     QtVariantPropertyManager::setValue(property, val);
@@ -142,8 +210,16 @@ void VariantPropertyManager::setAttribute(QtProperty *property,
         return;
     }
 
-    if (attribute == mSuggestionsAttribute && mSuggestions.contains(property))
-        mSuggestions[property] = val.toStringList();
+    if (mStringAttributes.contains(property)) {
+        if (attribute == mSuggestionsAttribute) {
+            mStringAttributes[property].suggestions = val.toStringList();
+            return;
+        }
+        if (attribute == mMultilineAttribute) {
+            mStringAttributes[property].multiline = val.toBool();
+            return;
+        }
+    }
 
     QtVariantPropertyManager::setAttribute(property, attribute, val);
 }
@@ -153,8 +229,10 @@ void VariantPropertyManager::initializeProperty(QtProperty *property)
     const int type = propertyType(property);
     if (type == filePathTypeId())
         mValues[property] = Data();
+    else if (type == tilesetParametersTypeId())
+        mValues[property] = Data();
     else if (type == QVariant::String)
-        mSuggestions[property] = QStringList();
+        mStringAttributes[property] = StringAttributes();
 
     QtVariantPropertyManager::initializeProperty(property);
 }
@@ -162,10 +240,9 @@ void VariantPropertyManager::initializeProperty(QtProperty *property)
 void VariantPropertyManager::uninitializeProperty(QtProperty *property)
 {
     mValues.remove(property);
-    mSuggestions.remove(property);
+    mStringAttributes.remove(property);
     QtVariantPropertyManager::uninitializeProperty(property);
 }
 
 } // namespace Internal
 } // namespace Tiled
-
